# -*- coding: utf-8 -*-
"""
===============================================================================
    Copyright © 2021  Kouadio K.Laurent
    
    This file is part of pyCSAMT.
    
    pyCSAMT is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    pyCSAMT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with pyCSAMT.  If not, see <https://www.gnu.org/licenses/>.

===============================================================================

.. _module-Plot_Utils:: `pycsamt.utils.plot_utils` 
    :synopsis: helpers functions for visualization 
    
Created on Tue Dec 29 19:18:44 2020

@author: @Daniel03
"""
import re , os, warnings
import numpy as np
from pycsamt.etc.infos import _sensitive as SB
import matplotlib as mpl 
import matplotlib.cm as cm 
import matplotlib.pyplot as plt
from pycsamt.utils import exceptions as CSex 
from pycsamt.utils.decorator import deprecated, redirect_cls_or_func


def share_props_for_each_plot(number_of_plot = 3 ,  **kwargs):
    """
    Function to set properties for each plot. Easy to customize line and 
    markers. Function can add other properties which are not in kwargs.keys(). 
    It will set according the number of subplotsplots 
    we assume subplot are define on one columns 
    
    :param number_of_plot: number of subsplot you want show.
    :type number_of_plot: int 
    
    :returns: dictionarry of labels and properties. 
    :rtype: dict 
    
    """
    
    def set_multiple(labelprops, default=None): 
        """ 
        set multiple labels 
        
        """
        if not isinstance(labelprops, (tuple, list, str)):
            labelprops=[labelprops] 

        if  isinstance (labelprops, str ): 
            if (labelprops.find(',') > 0) and (labelprops.find(':') >0): 
                labelprops=labelprops.replace(',',":")
                labelprops = labelprops.split(':')
            elif labelprops.find(',') > 0:labelprops= labelprops.split(',')
            elif labelprops.find(':') >0 : labelprops = labelprops.split(':')
            else :labelprops=[labelprops]

        if default is not None :  lbproperty = [default 
                                            for ll in range(number_of_plot)]
        else : lbproperty = [labelprops[0] for ll in range(number_of_plot)]

        if len(labelprops)  < number_of_plot : 
           lbproperty[:len(labelprops)]=labelprops
        elif len(labelprops)  > number_of_plot :
            lbproperty = labelprops[:number_of_plot]
        elif len(labelprops)  ==  number_of_plot : 
            lbproperty = labelprops
  
        return lbproperty 

    # linewidth =
    linewidth=set_multiple(labelprops=kwargs.pop('lw', 2.) , default=2.)
    
    linestyle=set_multiple(labelprops=kwargs.pop('ls',2.) , default=2.)
    color=set_multiple(labelprops=kwargs.pop('color', 'k') , default='k')
    
    marker=set_multiple(labelprops=kwargs.pop('marker', 'o'), default='o')
   
    markerfacecolor=set_multiple(labelprops=kwargs.pop(
        'markerfacecolor', 'k'), default='k')
    markeredgecolor =set_multiple(labelprops=kwargs.pop(
        'markeredgecolor','k') , default='k')
    
    rotate_angle_xticklabels=set_multiple(labelprops=kwargs.pop (
        'xtick_label_rotation',45.), default=45.)
    rotate_angle_yticklabels=set_multiple(labelprops=kwargs.pop(
        'ytick_label_rotation',None), default=None)
    
    x_ticks_labelsize =set_multiple(labelprops=kwargs.pop(
        'xtick_label_size', 12.) , default=12.)
    y_ticks_labelsize=set_multiple(labelprops=kwargs.pop(
        'ytick_label_size', 12.) , default=12.)

    # dict of labelproperties.
    kws ={'lw':linewidth, 
          'ls':linestyle, 
          'marker':marker, 
          'color':color, 
          'markerfacecolor':markerfacecolor, 
          'markeredgecolor':markeredgecolor, 
          'xtick_label_rotation':rotate_angle_xticklabels, 
          'ytick_label_rotation':rotate_angle_yticklabels, 
          'xtick_label_size':x_ticks_labelsize , 
          'ytick_label_size':y_ticks_labelsize}

    #can add other properties which are not in kwargs .keys ()
    for keys in list(kwargs.keys()):
        if keys not in list(kws.keys()):
            kws.__setitem__(keys, set_multiple(
                labelprops=kwargs[keys], default=kwargs[keys]))
        
    
    return kws

def find_path (path =None, ptol =0.7):
    """
    check path and return filepath , edipath or jpath .
    
    :param path: full  path  to `edi`, `avag` or `j` file or directory 
    :type path: str 
    
    :param ptol:  tolerance that given by the program to judge if the number 
                    of typical file [EDI|J] 
                    to declare as path found is either "edipath" or "jpath" 
                    if none ,return None . less or equal to 1.
                    
    :type ptol: float 
    
    :returns: specific path 
    :rtype: str 
    """
    if path is None : raise CSex.pyCSAMTError_plot_tip(
            'Can not find path to read.please provided a datapath . ')
    if path is not None : 
        if os.path.isfile (path) is True : return 'isfile'
        elif os.path.isdir(path) is True :
            if os.listdir(path) is not None : 
                ex = [file for file in os.listdir(path) 
                      if os.path.splitext(file)[1] =='.edi']
                if len(ex)/len(os.listdir(path))>= ptol :return 'edipath'
                elif len(ex)/len(os.listdir(path)) < ptol : 
                    m=[]
                    try : m= [file for file in os.listdir(path)
                              if SB.which_file(
                                      filename = os.path.join(
                                          path, file)) =='j' ]
                    except : pass 
                    if len(m)/len(os.listdir(path)) >= ptol :return 'jpath'
                        
        return
    
def station_id (id_): 
    """ 
    From id get the station  name as input  and return index `id`. 
    Index starts at 0.
    
    :param id_: name of the station or index . Be sure to hve the station name 
        containing the letter `S` which mean `site`. 
    :return: station index. If the list `id_` is given 
            will return the tuple.
    """   
    def _sid (n_):
        try : 
            n_=int(n_)
        except:
            if n_.lower().find('s')>=0 : 
                if n_.lower().count('s')==1: 
                    n_=n_.lower().replace('s', '')
                    try : 
                        n_=int(n_) 
                    except: 
                        raise CSex.pyCSAMTError_station(
                            f'Station `{n_}` is wrong! Please provided '
                            'the right id.')
        return n_
       
    if isinstance(id_, str): id_= _sid(id_)
    elif isinstance(id_, (float, int)):
        id_=int(id_)-1
        if id_<0:
            id_=1
    else: 
        try: 
            iter(id_)
        except: 
            raise CSex.pyCSAMTError_station(
                f'No iterable station `{id_}` found.')
        else: 
            id_= tuple([_sid (iid ) for iid in id_])
        
    return id_

                
           
def get_stationid (stations ,  station_id) :  
    """
    Tip to get station id from user by input either integer or station name .
    
    :param stations:  list of stations known
    :type stations: list
    
    :param station_id: staion expect to plot.
    :type station_id: list, str, or int 
            
    :returns: constructed list for plotting
    :rtype: array_like 
    
    :Example:
        
        >>> from pycsamt.utils import plot_utils as punc
        >>> teslist = ['S{0:02}'.format(ii) for ii in range(23)]
        >>> ss = punc.get_stationid (stations=teslist ,  station_id=('S04',13))
        >>> print(ss)
    """                
    
    sid=[]
    if not isinstance(station_id, (list, tuple)) : 
        station_id =[station_id]
    for iid in station_id : 
        if isinstance (iid, str):
           for ss , stn in enumerate(stations ): 
               if iid.lower()  ==stn.lower() : sid.append(stn)
        if isinstance(iid, (int, float)): 
            if int(iid) > len(stations): 
                raise CSex.pyCSAMTError_plot_tip(
                    'maximum station number is <{0}>.'
                'Can not plot beyond this limit.'
                ' Your stations go to <{1},...,{2}>'.\
                    format(len(stations), stations[0], stations[-1]))
                  
            sid.append(stations[int(iid) -1])

    return sid 
    
def get_frequency_id (freq_array , frequency_id ): 
    """
    function to get id of frequency . Frequency to plot
    
    :param freq_array:  array of frequency
    :type freq_array:  nd.array,1 
    
    :param frequency_id:  frequency to plot .
    :type frequency_id: list or float  
    
    :returns: new close frequency id
    :rtype: float|list
    """
    freqID =[] 
    if isinstance(freq_array, list) : 
        freq_array=np.array(freq_array)
    if isinstance(frequency_id, (str, float, int)):
        frequency_id =[frequency_id]
    
    for freqid in frequency_id:
        if isinstance(freqid ,str) :
            try : freqid =np.float(freqid)
            except : raise CSex.pyCSAMTError_plot_tip(
                    'Frequency value must be an integer of float not str ' )
        if freq_array.min() > freqid > freq_array.max() : 
            warnings.warn ('Can not find the id <{0}>. '
                           'ID provided is out of the range.Frequency trend'
                           ' to: <{1}|{2}Hz to {3}|{4}Hz>.'.\
                            format(freqid, np.where(
                                freq_array==freq_array.min()),freq_array.min(),
                                   np.where(freq_array==freq_array.max()),
                                   freq_array.max()))
            raise CSex.pyCSAMTError_plot_tip(
                'frequency value <{0}Hz> is out of the range'.format(freqid))
        freqID.append(freqid)
        
    return freqID

def getcloser_frequency(freq_array, frequency_id): 
    """ chek whether the frequency value given is in frequency range 
    if not found the closest value"""
    try : 
        frequency_id= float(frequency_id)
    except: 
        raise CSex.pyCSAMTError_frequency(
            f'Frequency must be a float value not {frequency_id}.')
    else: 
        indexf = np.where(freq_array == frequency_id )[0]     
        if len(indexf)>1:
            indexf = indexf[0] # take the fist values 
        elif len(indexf) ==0: 
            sustract_freq = np.abs(freq_array - frequency_id)
            indexf = np.where(sustract_freq==sustract_freq.min())[0]
            frequency_id = freq_array[indexf]
           
    return int(indexf), float(frequency_id)
                
        
    
def slice_matrix (base_matrix , freq_array, doi=2000):
    """
    Function get a matrix and give new matrice slice from limit value  

    :param base_matrix: arrays (yaxis lenghth, station_length)
    :type base_matrix: ndarray
    
    :param freq_array:  frequency array range 
    :type freq_array: ndarray,1 
    
    :param doi: expect to be the depth of investigation from 
                    which data muts be selected in `m` or `km` 
    :type doi:float 
    
    :returns: matrix sliced according to doi
    :rtype: ndarray 
    """
    fm=0
    if isinstance (doi, str):
        try : doi =float(doi)
        except :
            if doi.find('km')>=0 :
                ndoi = doi.replace('km','')
                fm=1
            elif doi.find('m')>=0 :ndoi = doi.replace('m','')
            try : doi =float(ndoi)
            except :
                warnings.warn ('Can not convert <{0}> into float number. '\
                               'Depth of investigation value must'
                               ' be float number.'.format(doi))
                raise CSex.pyCSAMTError_plot_tip(
                    'Value provided <{0}> must be float'
                    ' number not <{1}>'.format(doi, type(doi)))
            else :
                if fm==1 : doi *= 1e3   # convert into m
        
    doi =float(doi) # for consistency , convert to float
    indexMatrix = np.where(base_matrix >= doi)[0][0] 

    return base_matrix[:indexMatrix, ::], freq_array[:indexMatrix], doi 
    
def delineate_curve ( dict_loc , value , atol=0.2, replace_value = np.nan): 
    """
    function to delineate value of rho and phase .
    
    :param dict_loc:  dictionnary composed of keys = stations id and values
    :type  dict_loc: dict 
    
    :param value: value to delineate curve . for single value. 
    :type  value: float |list 
    
    :param atol: tolerance parameter <=1 . Most the param is closest to 0 , 
                 most the selected data become severe.default is 0.2
    :type atol: float   
                     
    :param replace_value: could be None or np.nan , Default is np.nan
    :type replace_value: float or else
                    
    :returns: dict of delineate data
    :rtype: dict

    :Example:
        
        >>> from pycsamt.utils import plot_utils as punc
        >>> ts = np.array([0.1, 0.7, 2, 3, 8, 1000, 58,55, 85, 18])
        >>> to =np.array([51, 78, 0.25, 188, 256, 7])
        >>> tt ={'S00': ts, 'S01':to, 'S02': np.array([0, 5, 125, 789])}
        >>> print(punc.delineate_curve(dict_loc = tt, 
                                       value=[50,70], atol = 0.1))
    """

    if isinstance(value, (str, int, float)): 
        try : value =float(value)
        except: 
            warnings.warn('Delineation value is float '
                          'number not <{0}>'.format(type(value)))
            raise CSex.pyCSAMTError_plot_tip(
                'Input delineation value is unacceptable.'
                ' Please enter a float number.')
        else : value =[value]
        
    elif isinstance(value, (list, tuple())): 
        if type(value) ==tuple : value =list(value)
        try : 
            value =sorted([float(ss) for ss in value]) 
        except : 
            warnings.warn('Delineation value is'
                          ' float number not <{0}>'.format(type(value)))
            raise CSex.pyCSAMTError_plot_tip(
                'Input delineation value is '
                'unacceptable. Please enter a float number.')
    try : 
        atol =float(atol)
        if atol > 1  or atol <0: 
            warnings.warn ('Tolerance value is <=1 (less or egal than 1). '
                           'Please enter again a new value.')
            raise CSex.pyCSAMTError_plot_tip(
                'Tolerance value is assumed to be'
                ' 0<= atol <=1 .Please check your value.')
    except : 
        warnings.warn('Tolerance value <{0}> provided is wrong.'
                      ' Must be a float number 0<= atol <=1 '
                      ' not a <{1}>'.format(atol, type(value)))
        raise CSex.pyCSAMTError_plot_tip(
            'Unacceptable type tolerance value.'
            ' Please enter a float number 0<= atol <=1.')
    
    # loop dictionnary an dkey value .
    delineate_dict ={}

    for value_to_find  in value :
        delineate_data_curve = []
        for keys, value_in_dict in sorted(dict_loc.items()):
            if value_to_find in value_in_dict: 
                 delineate_data_curve.append(value_to_find)      
            elif  value_to_find not in value_in_dict: 
                tem=[]
                min_value = value_to_find - atol * value_to_find 
                max_value = value_to_find + atol * value_to_find
                for new_turn_value  in value_in_dict :
                    if min_value <= new_turn_value <= max_value  : 
                        tem.append(new_turn_value)       
                if tem==[] : 
                    delineate_data_curve.append(replace_value)       
                elif tem !=[]: 
                    dx = np.abs(np.array(tem) - value_to_find)
                    minimum = dx.min()
                    valueabs = np.array(tem)[np.where (dx == minimum)] 
                    delineate_data_curve.append(float(valueabs) )
       
        delineate_dict['{0}'.format(value_to_find)] =np.array(
            delineate_data_curve)
  
    return delineate_dict
                    
def delineate_sparseMatrix (dict_loc, delineate_dict, replace_value =np.nan ):
    """
    Build from delineate dict a matrix according to frequency length . 
    value doesnt exist in the delineate dict will be repalce by 
    replacevalue . Default is `nan`.
    
    :param delineate_dict: delineation value 
    :type delineate_dict: dict
    
    :param dict_loc: dictionnary composed of keys = stations id and values
    :type dict_loc: dict 
    
    :param replace_value: value to replece other value  
        like build a sparse matrix 
    :type replace_value: float 
    
    :returns: dit of sparse matrix 
    :rtype: dict 
    """
    dictMatrix ={}
    dict_loc =sorted(dict_loc.items())
    for dd, ( keys , dvalues) in enumerate(dict_loc): 
        
        for kk, (keyi, vv) in enumerate(sorted(delineate_dict.items())[dd]): 
        #     if vv[dd] !=np.nan : 
            os= np.where(dvalues==vv[dd], dvalues, replace_value)
            dictMatrix[keyi] = os
            if vv[dd] == replace_value : 
                dictMatrix[kk]= np.repeat(replace_value, dvalues.shape[0])
            
    return dictMatrix

def resetting_ticks ( get_xyticks,  number_of_ticks=None ): 
    """
    resetting xyticks  modulo , 100
    
    :param get_xyticks:  xyticks list  , use to ax.get_x|yticks()
    :type  get_xyticks: list 
    
    :param number_of_ticks:  maybe the number of ticks on x or y axis 
    :type number_of_ticks: int
    
    :returns: a new_list or ndarray 
    :rtype: list or array_like 
    """
    if not isinstance(get_xyticks, (list, np.ndarray) ): 
        warnings.warn ('Arguments get_xyticks must be a list'
                       ' not <{0}>.'.format(type(get_xyticks)))
        raise CSex.pyCSAMTError_plot_tip(
            '<{0}> found. "get_xyticks" must be a list'
            ' or (nd.array,1).'.format(type(get_xyticks)))
    
    if number_of_ticks is None :
        if len(get_xyticks) > 2 : 
            number_of_ticks = int((len(get_xyticks)-1)/2)
        else : number_of_ticks  = len(get_xyticks)
    
    if not(number_of_ticks, (float, int)): 
        try : number_of_ticks=int(number_of_ticks) 
        except : 
            warnings.warn('"Number_of_ticks" arguments'
                          ' is the times to see the ticks on x|y axis.'\
                          ' Must be integer not <{0}>.'.format(
                              type(number_of_ticks)))
            raise CSex.pyCSAMTError_plot_tip(
                '<{0}> detected. Must be integer.')
        
    number_of_ticks=int(number_of_ticks)
    
    if len(get_xyticks) > 2 :
        if get_xyticks[1] %10 != 0 : 
            get_xyticks[1] =get_xyticks[1] + (10 - get_xyticks[1] %10)
        if get_xyticks[-2]%10  !=0 : 
            get_xyticks[-2] =get_xyticks[-2] -get_xyticks[-2] %10
    
        new_array = np.linspace(get_xyticks[1],
                                get_xyticks[-2], number_of_ticks )
    elif len(get_xyticks)< 2 : 
        new_array = np.array(get_xyticks)
 
    return  new_array
        
        
def resetting_colorbar_bound(cbmax , cbmin, number_of_ticks = 5,
                             logscale=False): 
    """
    Function to reset colorbar ticks more easy to read 
    
    :param cbmax: value maximum of colorbar 
    :type cbmax: float 
    
    :param cbmin: minimum data value 
    :type cbmin: float  minimum data value
    
    :param number_of_ticks:  number of ticks should be 
                            located on the color bar . Default is 5.
    :type number_of_ticks: int 
    
    :param logscale: set to True if your data are lograith data . 
    :type logscale: bool 
    
    :returns: array of color bar ticks value.
    :rtype: array_like 
    """
    def round_modulo10(value): 
        """
        round to modulo 10 or logarithm scale  , 
        """
        if value %mod10  == 0 : return value 
        if value %mod10  !=0 : 
            if value %(mod10 /2) ==0 : return value 
            else : return (value - value %mod10 )
    
    if not(number_of_ticks, (float, int)): 
        try : number_of_ticks=int(number_of_ticks) 
        except : 
            warnings.warn('"Number_of_ticks" arguments'
                          ' is the times to see the ticks on x|y axis.'\
                          ' Must be integer not <{0}>.'.format(
                              type(number_of_ticks)))
            raise CSex.pyCSAMTError_plot_tip(
                '<{0}> detected. Must be integer.')
        
    number_of_ticks=int(number_of_ticks)
    
    if logscale is True :  mod10 =np.log10(10)
    else :mod10 = 10 
       
    if cbmax % cbmin == 0 : 
        return np.linspace(cbmin, cbmax , number_of_ticks)
    elif cbmax% cbmin != 0 :
        startpoint = cbmin + (mod10  - cbmin % mod10 )
        endpoint = cbmax - cbmax % mod10  
        return np.array([round_modulo10(ii) 
                         for ii in np.linspace(startpoint,
                                               endpoint, number_of_ticks)])
    
    
def slice_csamt_matrix ( block_matrix , station_offsets, 
                        depth_offsets, offset_MinMax =(0, 1000), 
                        doi='2000m' ) : 
    """
    Using Wannamaker FE elements mesh to define rho matrix blocks ,
    need after inversion to slice the model 
    resistivity according the offset and depth we need . 
    This function is easy tool to slice matrix and to keep
    the part we need . station offset , depth and model resistivity 
    
    Parameters
    -----------
        * block_matrix : ndarray(station_offsets.shape[0], 
                    matrix of station depth Resistivity model
                    depth_offsets.shape[0])
        
        * depth_offset : array_like  
                   depth of investigation after generating by mesh 
                   file :>z_nodes . 
                   
        * station_offsets : array_like 
                  station _offsets : offset generate by mesh_file :>x_nodes .
                  
        * offset_MinMax : tuple  
              the interval of data to keep . eg if station location start by 0 : 
                  off[0] = min and off[-1]=max (min, max):--> index 0 :
                  minimum value of station location -->index 1 :
                      maximum value of station location
                  *default* is (0,1000)
                        
        * doi : str , float 
                investigation depth ,  migth be [m|km]. 
                If value is provided is float number , it might take value 
                as a default unit 'meter'. i.e : 1000="1000m"
                
    Returns
    ---------
        tuple 
          new sliced station offset , new sliced depth offset  ,
          new_matrix block , 
    """
    def depth_of_investigation(doi): 
        """
        :returns: doi in meter whether you provide value in kilometer 
        :rtype:float 
    
        """
        fm=0
        if isinstance (doi, str):
            try : doi =float(doi)
            except :
                if doi.find('km')>=0 :
                    ndoi = doi.replace('km','')
                    fm=1
                elif doi.find('m')>=0 :ndoi = doi.replace('m','')
                try : doi =float(ndoi)
                except :
                    warnings.warn ('Can not convert <{0}> into float number. '
                                   'Depth of investigation '
                                   'value must be float number.'.format(doi))
                    raise CSex.pyCSAMTError_plot_tip(
                        'Value provided <{0}> must be float'
                        ' number not <{1}>'.format(doi, type(doi)))
                else :
                    if fm==1 : doi *= 1e3   # convert into m
            
        return float(doi) # for consistency , convert to float
    
    doi= depth_of_investigation(doi)
    
    # --> get  the depth  depth indice 
    if isinstance(station_offsets, (list, tuple)) :
        station_offsets=np.array(station_offsets)
    if isinstance(depth_offsets, (list, tuple)) :
        depth_offsets= np.array(depth_offsets)
    if isinstance(offset_MinMax, (list, tuple)) : 
        offset_MinMax= np.array(offset_MinMax)
    
    #offsets min and maximal station offsets and depth 
    offset_min,  offset_max = offset_MinMax.min(), offset_MinMax.max()#
    offset_start, offset_end = station_offsets.min(), station_offsets.max()    
    
    #check wheren the value provide is on the offset 
    if offset_min < offset_start : 
        warnings.warn ('Povided minimal offset is out of the range '
                       '!Offset minimal for plottig is ={0}'.format(
                           offset_start))
        raise CSex.pyCSAMTError_plot_tip(
            'Povided minimal offset out'' of the range '
            '!Offset minimal for plottig is ={0}'.format(offset_start))
    if offset_max > offset_end  : 
        warnings.warn ('Povided maximal offset is out of the range '
                '!Offset maximal for plottig is ={0}'.format(offset_end))
        raise CSex.pyCSAMTError_plot_tip(
            'Povided maximal offset is out of the range !Offset'
            ' maximal for plottig is ={0}'.format(offset_end))
    
    if doi > depth_offsets.max() : 
        mess ='depth <{0}> is out of depth range. Maximum depth for'\
            '  is={1}.'.format(doi, depth_offsets.max())
        warnings.warn(mess)
        raise CSex.pyCSAMTError_plot_tip(mess)
    # make sure that station offset and and depth are not  in decrease oder
    sflip,dflip=0,0
    if depth_offsets[0] > depth_offsets[-1] : 
        depth_offsets =depth_offsets [::-1]
        sflip =1
    if station_offsets [0] > station_offsets [-1] : 
        station_offsets= station_offsets[::-1]
        dflip=1
        
    # check whether index values are present on depth array 
    ind_depth_s, = np.where(depth_offsets==0.)
    ind_depth_e, = np.where(depth_offsets==doi)

    #if not : 
    if len(ind_depth_s)==0 : 
        for ii, dd in enumerate(depth_offsets): 
                if dd > 0. : 
                    dmax , dmin = abs(dd-0.), abs(depth_offsets[ii-1]) 
                    if dmax < dmin : 
                        ind_depth_s =ii 
                        break
                    else :
                        ind_depth_s = ii-1
                        break
    if len(ind_depth_e) ==0 : 
        for ji, de in enumerate(depth_offsets): 
                if de > doi : 
                    dmax , dmin = abs(de-doi), abs(depth_offsets[ji-1]-doi) 
                    if dmax < dmin : 
                        ind_depth_e = ji 
                        break
                    else : 
                        ind_depth_e = ji-1
                        break

      #--> get for the station offset
    #checker whether station index are presents 
    ind_offs_s, = np.where(station_offsets==offset_min)
    ind_offs_e, = np.where(station_offsets==offset_max)
     
    # if not : 
    if len(ind_offs_s)==0 : 
        for jj, offs in enumerate(station_offsets): 
                if offs > offset_min : 
                    #claculate the difference betwen 
                    maxoff = abs(offs- offset_min) 
                    minoff = abs(station_offsets[jj-1]- offset_min)
                    if maxoff < minoff:
                        ind_offs_s = jj
                        break
                    else : 
                        ind_offs_s = jj-1
                        break
    if len(ind_offs_e) ==0 : 
         for je, offs in enumerate(station_offsets): 
                if offs> offset_max : 
                    #claculate the difference betwen 
                    maxoff = abs(offs-offset_max) 
                    minoff = abs(station_offsets[je-1]-offset_max)
                    if maxoff < minoff: 
                        ind_offs_e = je
                        break
                    else :
                        ind_offs_e = je -1 
                        break
     
    # convert to integer and leave array for indexing 
    ind_depth_s, ind_depth_e = int(ind_depth_s), int(ind_depth_e) 
    ind_offs_s, ind_offs_e= int(ind_offs_s), int(ind_offs_e)
    
    # for add +1 for the last index so to take the last value
    ind_offs_e +=+1
    ind_depth_e +=1 
 
    new_station_offsets= station_offsets[ind_offs_s: ind_offs_e]
    new_depth_offsets= depth_offsets[ind_depth_s:ind_depth_e]

    # Flip indexing if values array are sorted in highest
    # to lower (depth even stations locations)
    if sflip ==1 :
        # flips station -index 
        ind_offs_s = - ind_offs_e
        ind_offs_e = - ind_offs_s
        new_station_offsets= new_station_offsets[::-1]
   

    if dflip ==1 : 
        # flips depth-index 
        ind_depth_s = - ind_depth_e
        ind_depth_e = - ind_depth_s
        new_depth_offsets=new_depth_offsets[::-1]
        

        
    new_block_matrix = block_matrix[
        ind_depth_s:ind_depth_e, ind_offs_s:ind_offs_e] 
    
    return   (new_station_offsets,  new_depth_offsets, new_block_matrix)
            
def controle_delineate_curve(res_deline =None , phase_deline =None ): 
    """
    fonction to controle delineate value given  and return value ceilling .
    
    :param  res_deline:  resistivity  value todelineate. unit of Res in `ohm.m`
    :type  res_deline: float|int|list  
    
    :param  phase_deline:   phase value to  delineate , unit of phase in degree
    :type phase_deline: float|int|list  
    
    :returns: delineate resistivity or phase values 
    :rtype: array_like 
    """
    fmt=['resistivity, phase']
 
    for ii, xx_deline in enumerate([res_deline , phase_deline]): 
        if xx_deline is  not None  : 
            if isinstance(xx_deline, (float, int, str)):
                try :xx_deline= float(xx_deline)
                except : raise CSex.pyCSAMTError_plot_tip(
                        'Value <{0}> to delineate <{1}> is unacceptable.'
                         ' Please ckeck your value.'.format(xx_deline, 
                                                            fmt[ii]))
                else :
                    if ii ==0 : return [np.ceil(np.log10(xx_deline))]
                    if ii ==1 : return [np.ceil(xx_deline)]
  
            if isinstance(xx_deline , (list, tuple, np.ndarray)):
                xx_deline =list(xx_deline)
                try :
                    if ii == 0 : xx_deline = [np.ceil(np.log10(
                            float(xx))) for xx in xx_deline]
                    elif  ii ==1 : xx_deline = [np.ceil(float(xx))
                                                for xx in xx_deline]
                        
                except : raise CSex.pyCSAMTError_plot_tip(
                        'Value to delineate <{0}> is unacceptable.'
                         ' Please ckeck your value.'.format(fmt[ii]))
                else : return xx_deline

def depth_of_investigation(doi): 
    """
     Depth of investigation converter 

    :param doi: depth of investigation  if value float is provided , 
                it will considered as default units in meter
    :type doi: str|float 
    
    :returns doi:value in meter
    :rtype: float           
    """
    fm=0
    if isinstance (doi, str):
        try : doi =float(doi)
        except :
            if doi.find('km')>=0 :
                ndoi = doi.replace('km','')
                fm=1
            elif doi.find('m')>=0 :ndoi = doi.replace('m','')
            try : doi =float(ndoi)
            except :
                warnings.warn ('Can not convert <{0}> into float number. '
                               'Depth of investigation value must be float'
                               ' number.'.format(doi))
                raise CSex.pyCSAMTError_plot_tip(
                    'Value provided <{0}> must be float number'
                    ' not <{1}>'.format(doi, type(doi)))
            else :
                if fm==1 : doi *= 1e3   # convert into m
        
    return float(doi) # for consistency , convert to float


def build_new_station_id (station_id , new_station_name ): 
    """
    Fonction to build new station id including station name 
    provided , if the length provided 
        doesnt match the length of station id 
        
    :param station id: new sites names 
    :type station id: list 
    
    :param mess:  message for debugging, Default is None 
    :type  mess: str  
             
    :Example:
        
        >>> from pycsamt.utils import plot_utils as punc
        >>> ts =[   28.,  200.,  400.,  600.,  800., 1000.,
                 1200., 1400., 1600., 1800., 1807.]
        >>> sto = ['S{0}'.format(i) for i in range(7)]
        >>> print(sto)
        >>> stn, fu =punc.build_new_station_id(station_id = sto, 
                                               new_station_name =ts)
        >>> print(stn, fu)
    """
    mess=None
    get_len_new_station_id =len(new_station_name)
    if not isinstance(new_station_name, (list, tuple, np.ndarray)) : 
                new_station_name=[str(new_station_name)] 
    else : 
        new_station_name=[str(stn) for stn in new_station_name] 
    if get_len_new_station_id <  len(station_id ): 
        mess ='Length of sites names provided ={0}. It doesnt match the'\
            ' length of  station id ={1}. new sites names will build'\
            ' includingyour sites names provided'.format(len(new_station_name)
                                                         , len(station_id ))
        warnings.warn (mess)

        
        station_id =  new_station_name + station_id[get_len_new_station_id:]
    
    elif get_len_new_station_id > len(station_id) : 
        mess ='Length of sites names provided ={0}. It doesnt match the '\
            'length of  station id ={1} new sites names will be reduced'\
            ' according the length of station id.'
        mess.format(len(new_station_name), len(station_id ))
        warnings.warn (mess)
        
        station_id = new_station_name[:len(station_id) ]

    else : station_id = new_station_name 
    
    return [str(id_) for id_ in station_id] , mess

def get_conductive_and_resistive_zone (data, site_names,
                                       purpose ='groundwater',**kws): 
    """
    function to get the probability of conductive and resistive zone .
    It is not absolutely True but give an overview of decison .
     It is not sufficient to declare that the zone is 
    favorable for any drill , but just work with probability 

    :param data: resistivity datata of survey area 
    :type data: ndarray 
    
    :param site_name: list of sites names 
    :type site_name: list 
    
    :param purpose: type of exploration , default is `groundwater` 
    :type purpose: str 
    
    :returns: report of exploration area 
    :rtype: str 
    """
    def fmt_text (data_text, fmt='~', leftspace = 3, return_to_line =77) : 
        """
        Allow to format report with data text , fm and leftspace 
        :param  data_text: a long text 
        :type  data_text: str  
            
        :param fmt:  type of underline text 
        :type fmt: str

        :param leftspace: How many space do you want before
                            starting wrinting report .
        :type leftspae: int 
        
        :param return_to_line: number of character to return to line
        :type return_to_line: int 
        """
    
        return_to_line= int(return_to_line)
        begin_text= leftspace *' '
        text= begin_text + '~'*(return_to_line +7) + '\n'+ begin_text
    
        ss=0
        
        for  ii, num in enumerate(data_text) : 
            if ii == len(data_text)-1 :          
                #text = text + data_text[ss:] + ' {0}\n'.format(fmt) 
                # take the remain and add return chariot 
                text = text+ ' {0}\n'.format(fmt) +\
                    begin_text +'~'*(return_to_line+7) +'\n'        
     
                break 
            if ss == return_to_line :                       
                if data_text[ii+1] !=' ' : 
                    text = '{0}- {1} \n {2} '.format( 
                        text,  fmt, begin_text + fmt ) 
                else : 
                    text ='{0} {1} \n {2} '.format(
                        text, fmt, begin_text+fmt ) 
                ss=0
            text += num    # add charatecter  
            ss +=1
    
        return text 
    
    # other functions arguments
    model_offsets=kws.pop('model_offsets', None)
    site_offsets = kws.pop('site_offsets', None)
    purpose_value = np.log10(
        np.mean(
                [3.1e2, 1e0]))  
    
    average_rho_data =data.mean(axis = 0) 
    mins= average_rho_data.min()
    imins = average_rho_data.argmin()
    maxs =average_rho_data.max()
    imaxs = average_rho_data.argmax()

    mean_set_= average_rho_data.mean()         
    
   
    print('---> Average Rho on survey area is = {0} {1}'.format(
        np.power(10, mean_set_), 'Ω.m.'))
    
    if  imins < len(site_names) :
        print('---> Probably very conductive zone is'
              ' = {0} with rho = {1} Ω.m.'.
              format(site_names[int(imins)], np.power(10,mins)))
    if  imaxs < len(site_names) :
        print('---> Probably very resistive zone'
              ' is = {0} with rho = {1} Ω.m.'.
              format(site_names[int(imaxs)], np.power(10, maxs))) 
    
    # make a ratio minimum value and maxmim ration 
    temcon, temres=[[] for nn in range(2)] 
    for jj, (index, iindex) in enumerate(zip (
            [imins, imaxs], [mins, maxs])):
        if index > len(site_names): 
            if model_offsets is None  or site_offsets is None: 
                mess =''.join(
                    [' !Could not find Occam 2D model resistivity value .', 
                     ' Could not provide a report. Too many minimum value ', 
                    'find ! Need Occam model resistivity data and ', 
                    'model station data to provide a report.'])
                warnings.warn(mess)
        
            icc, offs_value = find_closest_station(
                offset_indice= index , 
                model_offsets = model_offsets,
                site_offsets =site_offsets)
            if jj ==0 : 
                snn = site_names[int(icc)] 
                if snn not in temcon:   
                    temcon.append((snn, iindex)) 
                    
            if jj ==1 : 
                snn = site_names[int(icc)] 
                if snn not in temres:              
                    temres.append((snn, iindex))
        
    if len(temcon) >0 : 
        for stn, rhoval  in temcon : 
            if len(temcon)>1 : 
                print(
                    '--> {0} Probably  conductive zones '
                    'has been selected :'.format(len(temcon)))
            
            print('---> Probably very conductive zone '
                  'is = {0} with rho = {1} Ω.m.'.
                  format(stn, np.power(10,rhoval)))
                
            imins =site_names.index(stn) 
            mins = rhoval
    if len(temres) > 0: 
        for stn, rhoval in temres : 
            if len(temres)>1 : 
                print('--> {0} Probably  resistives '
                      'zones has been selected :'.format(len(temres)))
            print('---> Probably very resistive zone '
                  'is = {0} with rho = {1} Ω.m.'.format(stn,
                                                        np.power(10,rhoval)))
                  
                
            imaxs =site_names.index(stn)
            maxs = rhoval
   
            
    print('- However :')
    print('---> {0:<27} {1} {2}. '.format(
        'Minimum ratio is',  '=',round( mins/mean_set_,7)))
    print('---> {0:<27} {1} {2}.'.format(
        'Maximum ratio is',  '=', round(maxs/mean_set_, 7)))
    
    if purpose =='groundwater':
        p_ratio = (purpose_value / mins)*100        
        p_ratio_max, p_ratio_min  = np.log10(3.1e2)/mins, np.log10(1e0) / mins 
        pseudo_trust = (mean_set_/ mins) *100 
        if pseudo_trust >  np.sqrt(2)* p_ratio_max *100 : 
             mess = ''.join([' resistive medium and could be ',
                             '  hydrogeologically UNfavorable zone.' ])
             fm='However,'
             # mess = 'hydrogeologically UNfavorable'
             
        elif pseudo_trust < np.sqrt(2)* p_ratio_min *100  : 
            mess ='medium too conductive and must be aware of that zone.'
            fm='However,'
            
        else : 
             mess ='hydrogeologically favorable zone and could'\
                 ' be a right drilling point.' 
             fm=''
        
        print()
        print( '!IMPORTANT NOTE! :')
        bfreport =''.join(['This present report is',
                           '  for {0}  exploration purpose.',  
                           'If your survey purpose if far from',
                           ' {0} exploration , please ignore it.'])
        bfreport =bfreport.format(purpose)
        
        fttext = fmt_text(data_text = bfreport, fmt='-', leftspace=3)
        
        print(fttext)
        
        print( 'Report:')

        report=''.join([
            'In the case of {0} exploration, along the survey line,',
            ' site {1} seems to be the best conductive zone with reliability ',
            ' ratio estimates to  = {2} % compared to the threshold = {3} %.',
            ' {4} Site {1} is considered as {5}. '
                    ])
        report= report.format(purpose, 
                              site_names[int(imins)],
                              np.around(pseudo_trust, 2), 
                              np.around(p_ratio,2),
                              fm, 
                              mess)
        caution =''.join([
            'The present  report is NOT absolute. It might be',
            ' taken with cautiousness because in groundwater ', 
            ' exploration, many other parameters are considering',
            ' before a possible drilling decison. We hope in  ', 
            'the next release, introduce an intelligent drill ', 
            'estimator `StarGain` to give better precision of ',
            ' absolute point  with tolerance value by considering', 
            '  multiple other parameters. '])
        
        # generate a reading report 
        textr = fmt_text(data_text = report + caution )
        print(textr)

        
def fmt_text (data_text, fmt='~', leftspace = 3, return_to_line =77) : 
    """
    Allow to format report with data text , fm and leftspace 

    :param  data_text: a long text 
    :type  data_text: str  
        
    :param fmt:  type of underline text 
    :type fmt: str

    :param leftspae: How many space do you want before 
                    starting wrinting report .
    :type leftspae: int 
    
    :param return_to_line: number of character to return to line
    :type return_to_line: int 
    """

    return_to_line= int(return_to_line)
    begin_text= leftspace *' '
    text= begin_text + fmt*(return_to_line +7) + '\n'+ begin_text

    
    ss=0
    
    for  ii, num in enumerate(data_text) :  
        if ii == len(data_text)-1 :         
            #text = text + data_text[ss:] + ' {0}\n'.format(fmt) 
            # take the remain and add return chariot 
            text = text+ ' {0}\n'.format(fmt) + \
                begin_text +fmt*(return_to_line+7) +'\n'        
 
            break 
        if ss == return_to_line :                       
            if data_text[ii+1] !=' ' : 
                text = '{0} {1}- \n {2} '.format(
                    text,  fmt, begin_text + fmt ) 
            else : 
                text ='{0} {1} \n {2} '.format(text, fmt, begin_text+fmt ) 
            ss=0
        text += num    # add charatecter  
        ss +=1

    return text 

def find_closest_station( offset_indice ,  model_offsets , site_offsets )  : 
    """ 
    Get the indice of the closest offset 
    
    Parameters
    -----------
        * offset_indice: int 
             if the indix of the offset at the selected resistivity point. 
        * model_offset: array_like  
            is a large band of x_nodes resistivities  gnerated by mesh files 
        
        * sites_offsets: array_like 
             the data set offset from Occam Data file 
        
    Returns 
    --------
        indexoff : int
             index of data offset 
        get_offs : float
               value of the offset at that index 
    """
    offs_value= model_offsets[int(offset_indice)] 

    for  ii , iof in enumerate(site_offsets) : 
        if iof > offs_value :  
            minabs = abs(iof - offs_value) 
            try : 
                maxabs = abs (site_offsets[ii-1]-offs_value) 
            except : 
                maxabs= abs(iof -offs_value)            
            if minabs <= maxabs :  
                indexoff, get_offs  = ii, iof
                break
            else :  
                indexoff, get_offs = ii-1, site_offsets[ii-1] 
                break
    return indexoff, get_offs
    
def find_local_maxima_minima (array): 
    """
    function to find minimum local and maximum local  on array 
    
    :param data: value of array to find minima , maxima 
    :type data: array_like 
    
    :returns: tuple of index of minima  and maxima local index and 
             array of minima maxima  value
    :rtype: array_like
    
    :Example: 
        
        >>> from pycsamt.utils import func_utils as func
        >>> from pycsamt.utils import plot_utils as punc
        >>> ts =np.array([2, 3, 4, 7, 9, 0.25, 18, 28, 86, 10, 5])
        >>> te=np.array([0.2, .3, 18, 1.6, 0.2, 0.6, 0.7, 0.8, 0.9, 1., 23.])
        >>> th =func.concat_array_from_list([ts, te], concat_axis=1)
        .. tth =th.T
        ... print(tth)
        ... print(punc.find_local_maxima_minima(tth[0]))
    """  
    localmm, indexlmm=[[] for ii in range(2)]
    
    error_mess ='Can not convert value to float number. Arry to find locals'\
                ' minmum and maximum must be an aray of float numbers.'
    if isinstance(array, (list, tuple)): # put on list 
        try :  array=np.array([float(ss) for ss in array])
        except : raise CSex.pyCSAMTError_parameter_number(error_mess)
            
    if isinstance(array, (float, str, int)):
        try :
            array =np.array([float(array)])
        except : CSex.pyCSAMTError_inputarguments(error_mess)
    
    if len(array) == 1 : 
        return (np.array([0]), array )
    # get the initial value , use the initial at starting points        
    d0, f=array[0], 0        #... can be either local maximum or local minimum         
    localmm.append(d0)              
    indexlmm.append(0)       # append index of starting point d0
    for ii, value in enumerate(array):  # let go for maximum data 
            if value >= d0 : #0.3       # try to find maximum |speudo maximum 
                if f==2 :               #but get the minimum index when
                    localmm.append(array[ii-1])
                    indexlmm.append(ii-1)
                f=1
                d0=value                # reinitialize pseudo maximum value 
            elif value < d0 :           # # try to find minimum|speudo minimum 
                if f== 1 :              #but get the maximum index when
                    localmm.append(array[ii-1])
                    indexlmm.append(ii-1)
                    
                f=2
                d0=value              # # reinitialize pseudo minimum value 
            if value ==array[-1]:     # at the end take the last value to close 
                localmm.append(value)  # the loop whether is minimum or maximum 
                indexlmm.append(ii)
                
    return np.array(indexlmm) ,np.array(localmm)            
        
def average_rho_with_locals_minmax(array): 
    """
    How to compute mean value between local maxima and local minima 
    and keep locals minima and maxima value on the final data 
    
    :param array: data to compute the local minima and local maxima 
    :type array: array_like 
    
    :returns: array mean with data local value averaged 
    :rtype: array_like
    
    :Example: 
        
        >>> from pycsamt.utils import plot_utils as punc
        >>> mean1= punc.average_rho_with_locals_minmax(tth[0])
        >>> mean2= punc.average_rho_with_locals_minmax(tth[1])
        >>> print(mean1)
        >>> print(mean2)
    """
    tem=[]          # find the local and maxium local index
    indexmm, _= find_local_maxima_minima(array) 
    
    if len(indexmm) == 1 :              # once it as a singe value  
        return array[int(indexmm)]      # for consistency
    
    s_index =indexmm[1]                 #   next index as str
    
    for ii, index in enumerate(indexmm): #[ 0,  4,  5,  8, 10] #s_index = 4
        if index == indexmm[-1]:
            # put single value on array for conacatenation                         
            tem.append(np.array(array[index])) 
            break
        
        if ii !=0 :                     # start a counter at index >0
            s_index = indexmm[ii+1]
         # take the next index and substract the present index s_index 
        ndiff = s_index - index # ndiff = 4-0 (indexmm[1]-     
                                        #index (indexmm[0])) = 4 >1    
        if ndiff == 1 :
            tem.append(np.array(array[index]))   #add local minimum array[0]= 2
        # [0:4+1]take the mean between [index0 : index 9 +1]--> [0:5]
        if ndiff >1 : 
            mean_mm = array[index:s_index+1].mean() 
            #moy = array[ii :indexmm[ii+1]] = 
            #array[0]-array[5]= moy (array[2:5] 2:9 /mean
            tem.append(np.array(array[index]))
             # 25/5 =5 ( 5, 3) 2 5 5 5 9 (ndiff =4-0-1)=3
            nn=np.repeat(mean_mm, ndiff-1)
            tem.append(nn)
               
    return np.hstack(tuple(tem))

def average_rho_in_deeper (dep_array, rho_array, step_descent ) :
    """
    function to average rho in deep according to the value provided . In fact 
    averaged rho in shorter depth distance allow us to understand the 
    conductive zone. in approximately. The most conductive zone is detected as 
    the zone with lower resistivities values . But fixing values as averaged 
    rho , can build a specific strata that could match this zone .
    
    Parameters
    ------------
        * dep_array : array_like 
                the imaged depth (doi)
            
        * rho_array: array_like
                resistivity array  
            
        * step_descent : float 
                value to step descent 
    
    Returns
    ---------
        array_like 
             rho average for each station  # dep_averaged for each station

    :Example:
        
        >>> import numpy as np
        >>> from pycsamt.utils import plot_utils as punc
        >>> pseudo_depth=np.array([  0. ,  6. , 13. , 20.  ,29. , 39.,  49. ,
        ...                       59. , 69. , 89., 109. ,129., 149. ,179.,
        ...                  209. ,249. ,289., 339., 399., 459. ,529. ,609., 
        ...                  699., 799., 899., 999.])
        >>> pseudo_depth = np.arange(0, 1220, 20)
        >>> rho = np.random.randn(len(pseudo_depth))
        >>> rho_aver, dep_aver= average_rho_in_deep(dep_array=pseudo_depth,
        ...                                        rho_array=rho, 
        ...                                        step_descent=20.)
        >>> rho_2,dep_2 =   average_rho_in_deeper (dep_array= pseudo_depth,
        ...                                         rho_array=rho,
        ...                                         step_descent=1000) 
        >>>  print(pseudo_depth)
    """
    v,r , dm, rm=[[] for i in range(4)]
    value = step_descent
    
    for index , depth in enumerate(dep_array):
        # value less than step descent must be averaged 
        if depth <= step_descent :     
            v.append(depth)             # keep resistivities values onto list 
            r.append(rho_array[index])

        if depth > step_descent :       # if the next value is greater 
            if v !=[]:                  # than the previous one ccheck if kist 
                                        # is not enmpty
                # rebuild resistivities values with rho averaged 
                dm.append(np.repeat(np.array(v).mean(), len(v)))  
                rm.append(np.repeat(np.array(r).mean(), len(r)))
                #increment the next descent to step of descent 
                step_descent += value 
                # initialise new list by adding the index value greater one 
                v=[depth]       
                r=[rho_array[index]]
              
        if depth ==dep_array[-1]:
            # it length last value ==1 , means is the last value of depth
            if len(v)==1 :        
                dm.append(dep_array[index])
                rm.append(rho_array[index])
            elif len(v) !=1 :          # averaged the reamin rho values  
                dm.append(np.repeat(np.array(v).mean(), len(v)))
                rm.append(np.repeat(np.array(r).mean(), len(r)))
      

    return np.hstack(tuple(rm)),np.hstack(tuple(dm))


@deprecated("function deprecated , it's does not cover the total depth when "
            "last value is outside the model depth range. ")
@redirect_cls_or_func(average_rho_in_deeper,"Function replaced ,"
                      " it redirects to {average_rho_in_deeper}.")
def average_rho_in_deep (dep_array, rho_array, step_descent)   :
    """
    function to average rho in deep according to the value provided . In fact 
    averaged rho in smalled depth distance allow us to understand the 
    conductive zone. in approximated. The most conductive zone is detected 
    as the zone with lower resistivities values . But fixing values as averaged 
    rho , can build a specific strata that could match the zone .
    
    Parameters
    -----------
        * dep_array : array_like
                the imaged depth (doi)
        * resistivity : array_like 
                resistivity ate the station 
        * step_descent : float 
                value to step descent 
    Returns 
    --------
    array_like 
         rho average for each station # dep_averaged for each station
  
    :Example: 
        
        >>> from pycsamt.utils import plot_utils as punc
        >>> pseudo_depth = np.arange(0, 1201.,100)
        >>> print(pseudo_depth)
        >>> dep_aver= dep(dep_array=pseudo_depth, value=100)
        >>> print(dep_aver)
        >>> print(pseudo_depth.shape)
        >>> print(dep_aver.shape)
    """
    tem, rm=[[] for i in range(2)]
    step_descent  =depth_of_investigation(step_descent)
    
    if isinstance(step_descent, (str, int)): 
        try : 
            step_descent =float(step_descent)
        except : 
            raise CSex.pyCSAMTError_plot_geoinputargument(
                'Could not convert depth value ={} to float.'
                ' Please check your value.'.format(step_descent))
    
    
    if step_descent < dep_array.min(): 
        raise CSex.pyCSAMTError_inputarguments(
            'Value provided ={0} m is less than the minimum depth'
            ' ={1} m.'.format(step_descent, dep_array.min()))
    
    if step_descent > dep_array.max(): 
        raise CSex.pyCSAMTError_inputarguments(
            'Value provided is = {0} m is greater than maximum'
            ' depth ={1}m.'.format(step_descent, dep_array.max()))
        
    mm=0 
    _init_depth =step_descent
  
    for index, depth in enumerate(dep_array):  # index = 0

        if depth >= step_descent : 
            # if value is greater than the maximum depth for the fist time 
            #[0 :20] 20 is ouut : 0--19 
            dep_averaged = dep_array[mm:index].mean() 
            rho_averaged=  rho_array[mm:index].mean()
            tem.append(np.repeat(dep_averaged, index - mm)) #3-0=3
            rm.append(np.repeat(rho_averaged, index - mm)) 
            mm= index               # mm= 3 (20)
           
            step_descent += _init_depth    # value = 20+20 = 40
    
            # last value then check if value is modulo the init depth 
        if depth == dep_array[-1]:               
            # assume the last depth is less than the next assigned depth 
            if depth % _init_depth == 0 or depth < step_descent : 

                dep_averaged = dep_array[mm:].mean()
                rho_averaged=  rho_array[mm:].mean()
                tem.append(np.repeat(dep_averaged, index -mm +1)) 
                rm.append(np.repeat(rho_averaged, index -mm +1))
        

        
    return np.hstack(tuple(rm)), np.hstack(tuple(tem))
    

def get_station_id_input_resistivities(station_rho_value,
                                       number_of_layer=None): 
    """
    Get a special station input resistivities is much benefit and 
    more close to reality of plot . Indeeed , it take only the maximum 
    value of resistivities  below the site and the minimum , then cut out 7 
    Considering all the model data and choose the max resistivities and the
     minim resistivities to build automatic resistivities whom COULD
     match the deth is less sure .Geeting a input resitivities
     to aplom the site , give a merly and better interpretation. 

    :param station_rho_value: value of resistivities under the 
                                site thin the maximum depth 
    :type station_rho_value: array_like
    
    :param number_of_layer: number of layer to top to bottom.
    :type number_of_layer: int 
    """
    if number_of_layer is None : number_of_layer =7.
    if isinstance(number_of_layer, (list, tuple, np.ndarray)): 
        try : 
            number_of_layer =int(number_of_layer[0])
        except : 
            raise CSex.pyCSAMTError_plot_tip(
                'Could not convert number of  layer ={0} value'
                ' into integer.'.format(tuple(number_of_layer)))
            
    if isinstance(number_of_layer, (float, str)): 
        try : int(number_of_layer)
        except : CSex.pyCSAMTError_plot_tip(
                'Could not convert number of layer ={} '
                'to integer.'.format(number_of_layer))
        else : number_of_layer=int(number_of_layer)
        
    if isinstance(station_rho_value, (list, tuple)): 
        try : 
            station_rho_value= np.array(station_rho_value)
            if station_rho_value.dtype not in ['float', 'int']: 
                station_rho_value= station_rho_value.astype('float64')
                
        except : raise CSex.pyCSAMTError_plot_geoinputargument(
                'Could not convert resistivities values '
                'to float number. Please provide a right values.')
                  
                                                               
    
    # get maxrho and min rho 
    return np.linspace(station_rho_value.min(), station_rho_value.max(),
                       number_of_layer)
        
def build_resistivity_barplot(depth_values , res_values): 
    """
    Allow to build bar plot resistivity function of  investigation depth .
    
    Parameters
    ----------
        * depth_values : aray_like 
                model investigation depth 
                
        * res_values : array_like 
                model_resistivities at each depth values 
            
    Returns 
    ---------
        d : array_like 
            resistivity barplot depth .
        r : array_like 
               specific structure resistivities 
        sumd : float
            checker number that cover in fact the total depth. 
            this numbe must absolutely match the total depth .

    """
    mess = "".join(["Depth and resistivity arrays have no the same Length .",
                    " Length Depth is ={0} and length depth is = {1}".format(
                        len(depth_values), len(res_values))
                    ])
    if len(depth_values) !=len(res_values) : 
        warnings.warn(mess)
        raise CSex.pyCSAMTError_geodrill_inputarguments(
            'Error size. Depth and resistivity must have the same length.')
    
    # lop the resistivities array 
    v,r=[],[]
    # mm = res_values [0]
    for ii , res in enumerate(res_values):
        if res == res_values [-1] : 
            if res_values [ii] != res_values[-2] :  
                v.append(depth_values[-1])
                r.append(res)
                break
            if res == res_values [-2] : 
                # v[-1]= depth_values [-1]
                v.append(depth_values [-1])
                r.append(res)
                
        elif res !=res_values[ii+1] : 
            v.append(depth_values [ii])
            r.append(res)
            
    # thens substract the difference between depth values
    #so all the sum must be egal to the maximum depth 
    d=[v[0]]        # because the minimum depth start by 0 ,
                    #then append  first value depth 
    
    for dd , depth in enumerate(v): 
        if dd >0 : 
            d.append(depth - v[dd-1]) # 
            
    # check the sum if it covers the total depth .
    d= np.array(d)
    sumd = d.sum()
    # sanitize array and eliminate all negative values and zeros 
    d[d<0]=0.           # then sanitize 
    d =d[d!=0]
    
    return d, r, sumd

def annotate_tip(layer_thickness , layer_names): 
    """
    A tip to group text with the same resistivities one layer when the layer 
    are successively the same .
    
    :param layer_thickness: thickness of layer
    :type layer_thickness: array_like |list 
    
    :param layer_names: names of layers , geological structures names 
    :type layer_names: list or array_like 
    
    :returns: v ,  layer  thickness in depth 
    :rtype: array_like 
    
    :returns: ni ,  list : name of layer 
    :rtype: array_like 
  
    :Example: 
        
        >>> from pycsamt.utils import plot_utils as punc
        >>> rocks = ['Massive sulfide', 'Igneous rocks', 
                     'Igneous rocks', 'Igneous rocks',
             'Igneous rocks', 'Igneous rocks', 'Igneous rocks',
             'Igneous rocks', 
             'Massive sulfide', 'Igneous rocks', 'Igneous rocks',
             'Igneous rocks'] 
        >>> resprops = [0.0, 49.0,69.0,89.0,109.0,129.0,
                        149.0,179.0,249.0,699.0,799.0,899.0]
        >>> thickness, lnames = punc.annotate_tip(layer_thickness=resprops,
                                                  layer_names=rocks)
        >>>print(thickness)
        >>> print(lnames)
        >>> v, ni
        ...  [24.5, 149.0, 474.0, 799.0]
        ...  ['Massive sulfide', 'Igneous rocks', 'Massive sulfide',
              'Igneous rocks']
    """
    if len(layer_thickness)==1 : 
        return layer_thickness, layer_names
    if len(layer_thickness)==0 : 
        mess = 'None value is provided, Please specify at least '\
            'the thickness of one truth layer.'
        warnings.warn(mess)
        raise CSex.pyCSAMTError_plot_tip(mess)
    
    top = layer_thickness[0]
    bottom = layer_thickness[1]
    ni=[layer_names[0]]
    v=[]
    for ii, thickvalues in enumerate(layer_thickness): 
        if ii != 0 :
            if ii == len(layer_thickness)-1 :  
                if layer_names[ii].lower()== layer_names[ii-1].lower() : 
                    mean = top + (thickvalues - top )/2
                    v.append(mean)
                    
                if layer_names[ii].lower() != layer_names[ii-1].lower() :  
                    mean =top + (layer_thickness[ii-1]-top)/2
                    v.append(mean)
                    mean = layer_thickness[ii-1] + (thickvalues- layer_thickness[ii-1])/2  
                    v.append(mean)
                    ni.append(layer_names[ii])                
            
            elif layer_names[ii].lower() != layer_names[ii-1].lower() : 
                bottom = thickvalues
                mean = (bottom - top )/2 + top       
                v.append(mean)                 
                ni.append(layer_names[ii])              
                top =bottom                  


    return v, ni  
    
def plot_errorbar(ax, x_array, y_array, y_error=None, x_error=None,
                  color='k', marker='x', ms=2, ls=':', lw=1, e_capsize=2,
                  e_capthick=.5, picker=None, **kws):
    """
    convinience function to make an error bar instance
    @author: jpeacock-pr 
    
    Arguments:
    ------------
        **ax** : matplotlib.axes instance 
                 axes to put error bar plot on
    
        **x_array** : np.ndarray(nx)
                      array of x values to plot
                      
        **y_array** : np.ndarray(nx)
                      array of y values to plot
                      
        **y_error** : np.ndarray(nx)
                      array of errors in y-direction to plot
        
        **x_error** : np.ndarray(ns)
                      array of error in x-direction to plot
                      
        **color** : string or (r, g, b)
                    color of marker, line and error bar
                    
        **marker** : string
                     marker type to plot data as
                     
        **ms** : float
                 size of marker
                 
        **ls** : string
                 line style between markers
                 
        **lw** : float
                 width of line between markers
        
        **e_capsize** : float
                        size of error bar cap
        
        **e_capthick** : float
                         thickness of error bar cap
        
        **picker** : float
                     radius in points to be able to pick a point. 
        
        
    Returns:
    ---------
        **errorbar_object** : matplotlib.Axes.errorbar 
                              error bar object containing line data, 
                              errorbars, etc.
    """
    # this is to make sure error bars plot in full and not just a dashed line
    if x_error is not None:
        #        x_err_high = np.array(x_error)
        #        x_err_low = np.array(x_err_high)
        #        x_err_low[x_err_high>=x_array] = x_array[x_err_high>=x_array]*.9999
        #        x_err = [x_err_low, x_err_high]
        x_err = x_error
    else:
        x_err = None

    if y_error is not None:
        #        y_err_high = np.array(y_error)
        #        y_err_low = np.array(y_err_high)
        #        y_err_low[y_err_high>=y_array] = y_array[y_err_high>=y_array]*.9999
        #        y_err = [y_err_low, y_err_high]
        y_err = y_error
    else:
        y_err = None

    errorbar_object = ax.errorbar(x_array,
                                  y_array,
                                  marker=marker,
                                  ms=ms,
                                  mfc='None',
                                  mew=lw,
                                  mec=color,
                                  ls=ls,
                                  xerr=x_err,
                                  yerr=y_err,
                                  ecolor=color,
                                  color=color,
                                  picker=picker,
                                  lw=lw,
                                  elinewidth=lw,
                                  capsize=e_capsize,
                                  #            capthick=e_capthick
                                  **kws)
    return errorbar_object    
   

if __name__=='__main__':
    print(station_id(['S00', 'S04', 'S09', 'S2']))
#     RES=np.array([1.200000e-01, 1.200000e-01 ,1.200000e-01, 1.200000e-01, 1.200000e-01,
#              1.200000e-01, 1.200000e-01, 1.138263e+04, 1.138263e+04, 2.276514e+04,
#              3.414765e+04, 4.553016e+04 ,3.414765e+04, 2.276514e+04, 1.138263e+04,
#              1.138263e+04, 1.200000e-01, 1.200000e-01, 1.200000e-01, 1.200000e-01,
#              1.200000e-01, 1.200000e-01, 1.200000e-01, 1.138263e+04 ,2.276514e+04,
#              6.829517e+04])
#     pseudo_depth=np.array([  0. ,  6. , 13. , 20.  ,29. , 39.,  49. , 59. ,
#                            69. , 89., 109. ,129., 149. ,179.,
#                       209. ,249. ,289., 339., 399., 459. ,529. ,609., 699., 799., 899., 999.])
    
#     barT,bar_res,  sumd = build_resistivity_barplot(depth_values=pseudo_depth ,
#             res_values=RES)
    
 
#     rocks = ['Massive sulfide', 'Igneous rocks', 'Igneous rocks', 'Igneous rocks',
#              'Igneous rocks', 'Igneous rocks', 'Igneous rocks', 'Igneous rocks', 
#              'Massive sulfide', 'Igneous rocks', 'Igneous rocks', 'Igneous rocks'] 

#     resprops = [0.0, 49.0,69.0,89.0,109.0,129.0,149.0,179.0,249.0,699.0,799.0,899.0]
    
#     thickness, lnames = annotate_tip(layer_thickness=resprops, layer_names=rocks)


    

    

    
    

    
    
    
    


# -*- coding: utf-8 -*-
"""
===============================================================================
    Copyright © 2021  Kouadio K.Laurent
    
    This file is part of pyCSAMT.
    
    pyCSAMT is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    pyCSAMT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with pyCSAMT.  If not, see <https://www.gnu.org/licenses/>.

===============================================================================  

Created on Tue Oct 13 14:52:08 2020

@author: Daniel03

use : 
    for sql dataBase works 
    dico to sql_database . 

"""


# from datetime import datetime, time, date, timezone 



class Glob(object): 
    """
    Spaces of variables and fonctions pseudo-globales .
    dictionnary can be set outside the container class Glob,
    following the dicoT datastructuration . 
    
    :Example: 
        
        >>> value_DB =[('id_new', 'i', 'new_vision'),
        ...   ('infoTab','k','no comment'), 
        ...   ('collar','d','collarDH')]
        >>> Glob.dicoT.__setitem__('TableDB_set',value_DB )
        
    """
    
    
    # struture of database, dictionnary of Table and fields # 
    
    dicoT={ "key_table_one": [("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment')],
    
           "key_table_two": [("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment'),
                                ("main_f","k",'id_','type_of_value','posi','num_of_value','date_time',"primary key",'comment')]}
    



# -*- coding: utf-8 -*-
# https://machinelearningmastery.com/implement-backpropagation-algorithm-scratch-python/

# Backprop on the Seeds Dataset
from random import seed
from random import randrange
from random import random
from csv import reader
from math import exp

from pycsamt.geodrill.geoDB.sql_recorder import GeoDataBase 
from pycsamt.utils._csamtpylog import csamtpylog 
_logger =csamtpylog().get_csamtpy_logger(__name__)

class ANN: 
    """
    Implement ANN to predict layer name from the SQL database. Each database 
    columns is a database properties 
    Each column of Γ  is a database property considered as a vector γ 
    with N_Γ- number of rocks. The column of electrical property of rocks 
    (E_props) is composed of the representative chart of  Palacky (1988)
    and  the rock and mineral property classification of Slichter and 
    Telkes (1942). The E_props column can be considered as a vector
    γ_(E_props ). Moreover, the powerful tool of SQLite  allows to build a 
    sub-table of E_props containing the minimal (v-) and the maximum (v+)
    resistivity values of each rock straightforwardly linked to the main column
    E_props of  Γ.  For instance, according to (Palacky, 1988), the rock of 
    massive sulfides ranges between  0.01 Ω.m to 1 Ω.m, the saprolite  
    between 2 and 200 Ω. m and  according to  (Slichter and Telkes, 1942), 
    the gneiss ranges between 10 000 to 〖10〗^7 Ω.m and so on. 
      https://pubs.usgs.gov/tm/2006/11A02/
      https://pubs.usgs.gov/of/2003/ofr-03-420/ofr-03-420.html

    
    reference: 
        * Slichter, L. B., and M. Telkes, 1942, Electrical Properties of Rocks 
            and Minerals, Geological. (F. B. C. Spicer, J. F. Schairer, 
             and S. H. Cecil, eds.,): © 1942 Geological Society of America.
            https://doi.org/10.1130/SPE36-p299
        * Palacky, G. J., 1988, Resistivity Characteristics of Geologic 
            Targets: Geophysics, 3, 
            https://doi.org/52–129.10.1190/1.9781560802631.ch3 

    """
    def __init__(self, data=None, database =None, **kwargs): 
        self.data =data 
        self._database =database 
        
        
        for key in list(kwargs.keys()): 
            self.__setattr__(key, kwargs[key])
            
            
# Load a CSV file
def load_csv(filename):
	dataset = list()
	with open(filename, 'r') as file:
		csv_reader = reader(file)
		for row in csv_reader:
			if not row:
				continue
			dataset.append(row)
	return dataset

# Convert string column to float
def str_column_to_float(dataset, column):
	for row in dataset:
		row[column] = float(row[column].strip())

# Convert string column to integer
def str_column_to_int(dataset, column):
	class_values = [row[column] for row in dataset]
	unique = set(class_values)
	lookup = dict()
	for i, value in enumerate(unique):
		lookup[value] = i
	for row in dataset:
		row[column] = lookup[row[column]]
	return lookup

# Find the min and max values for each column
def dataset_minmax(dataset):
	minmax = list()
	stats = [[min(column), max(column)] for column in zip(*dataset)]
	return stats

# Rescale dataset columns to the range 0-1
def normalize_dataset(dataset, minmax):
	for row in dataset:
		for i in range(len(row)-1):
			row[i] = (row[i] - minmax[i][0]) / (minmax[i][1] - minmax[i][0])

# Split a dataset into k folds
def cross_validation_split(dataset, n_folds):
	dataset_split = list()
	dataset_copy = list(dataset)
	fold_size = int(len(dataset) / n_folds)
	for i in range(n_folds):
		fold = list()
		while len(fold) < fold_size:
			index = randrange(len(dataset_copy))
			fold.append(dataset_copy.pop(index))
		dataset_split.append(fold)
	return dataset_split

# Calculate accuracy percentage
def accuracy_metric(actual, predicted):
	correct = 0
	for i in range(len(actual)):
		if actual[i] == predicted[i]:
			correct += 1
	return correct / float(len(actual)) * 100.0

# Evaluate an algorithm using a cross validation split
def evaluate_algorithm(dataset, algorithm, n_folds, *args):
	folds = cross_validation_split(dataset, n_folds)
	scores = list()
	for fold in folds:
		train_set = list(folds)
		train_set.remove(fold)
		train_set = sum(train_set, [])
		test_set = list()
		for row in fold:
			row_copy = list(row)
			test_set.append(row_copy)
			row_copy[-1] = None
		predicted = algorithm(train_set, test_set, *args)
		actual = [row[-1] for row in fold]
		accuracy = accuracy_metric(actual, predicted)
		scores.append(accuracy)
	return scores

# Calculate neuron activation for an input
def activate(weights, inputs):
	activation = weights[-1]
	for i in range(len(weights)-1):
		activation += weights[i] * inputs[i]
	return activation

# Transfer neuron activation
def transfer(activation):
	return 1.0 / (1.0 + exp(-activation))

# Forward propagate input to a network output
def forward_propagate(network, row):
	inputs = row
	for layer in network:
		new_inputs = []
		for neuron in layer:
			activation = activate(neuron['weights'], inputs)
			neuron['output'] = transfer(activation)
			new_inputs.append(neuron['output'])
		inputs = new_inputs
	return inputs

# Calculate the derivative of an neuron output
def transfer_derivative(output):
	return output * (1.0 - output)

# Backpropagate error and store in neurons
def backward_propagate_error(network, expected):
	for i in reversed(range(len(network))):
		layer = network[i]
		errors = list()
		if i != len(network)-1:
			for j in range(len(layer)):
				error = 0.0
				for neuron in network[i + 1]:
					error += (neuron['weights'][j] * neuron['delta'])
				errors.append(error)
		else:
			for j in range(len(layer)):
				neuron = layer[j]
				errors.append(expected[j] - neuron['output'])
		for j in range(len(layer)):
			neuron = layer[j]
			neuron['delta'] = errors[j] * transfer_derivative(neuron['output'])

# Update network weights with error
def update_weights(network, row, l_rate):
	for i in range(len(network)):
		inputs = row[:-1]
		if i != 0:
			inputs = [neuron['output'] for neuron in network[i - 1]]
		for neuron in network[i]:
			for j in range(len(inputs)):
				neuron['weights'][j] += l_rate * neuron['delta'] * inputs[j]
			neuron['weights'][-1] += l_rate * neuron['delta']

# Train a network for a fixed number of epochs
def train_network(network, train, l_rate, n_epoch, n_outputs):
	for epoch in range(n_epoch):
		for row in train:
			outputs = forward_propagate(network, row)
			expected = [0 for i in range(n_outputs)]
			expected[row[-1]] = 1
			backward_propagate_error(network, expected)
			update_weights(network, row, l_rate)

# Initialize a network
def initialize_network(n_inputs, n_hidden, n_outputs):
	network = list()
	hidden_layer = [{'weights':[random() for i in range(n_inputs + 1)]} for i in range(n_hidden)]
	network.append(hidden_layer)
	output_layer = [{'weights':[random() for i in range(n_hidden + 1)]} for i in range(n_outputs)]
	network.append(output_layer)
	return network

# Make a prediction with a network
def predict(network, row):
	outputs = forward_propagate(network, row)
	return outputs.index(max(outputs))

# Backpropagation Algorithm With Stochastic Gradient Descent
def back_propagation(train, test, l_rate, n_epoch, n_hidden):
	n_inputs = len(train[0]) - 1
	n_outputs = len(set([row[-1] for row in train]))
	network = initialize_network(n_inputs, n_hidden, n_outputs)
	train_network(network, train, l_rate, n_epoch, n_outputs)
	predictions = list()
	for row in test:
		prediction = predict(network, row)
		predictions.append(prediction)
	return(predictions)

if __name__=='main':
    # Test Backprop on Seeds dataset
    seed(1)
    # load and prepare data
    filename = 'seeds_dataset.csv'
    dataset = load_csv(filename)
    for i in range(len(dataset[0])-1):
    	str_column_to_float(dataset, i)
    # convert class column to integers
    str_column_to_int(dataset, len(dataset[0])-1)
    # normalize input variables
    minmax = dataset_minmax(dataset)
    normalize_dataset(dataset, minmax)
    # evaluate algorithm
    n_folds = 5
    l_rate = 0.3
    n_epoch = 500
    n_hidden = 5
    scores = evaluate_algorithm(dataset, back_propagation, n_folds, l_rate, n_epoch, n_hidden)
    print('Scores: %s' % scores)
    print('Mean Accuracy: %.3f%%' % (sum(scores)/float(len(scores))))


# -*- coding: utf-8 -*-
"""
===============================================================================
    Copyright © 2021  Kouadio K.Laurent
    
    This file is part of pyCSAMT.
    
    pyCSAMT is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    pyCSAMT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with pyCSAMT.  If not, see <https://www.gnu.org/licenses/>.

===============================================================================

.. _module-Dispatcher::`pycsamt.ff.processing.callffunc`  
    :synopsis: Typical call files functions  
                call computations functions , AGSO and dispatch for purposes  
    
Created on Thu Nov 26 20:55:39 2020

@author: @Daniel03

"""
import os,warnings 
# import warnings
import numpy as np
import pandas as pd
from pycsamt.utils import exceptions as CSex
from pycsamt.utils._csamtpylog import csamtpylog
from pycsamt.utils import func_utils as func
_logger=csamtpylog.get_csamtpy_logger(__name__)



#*** file from _geocodes folder :AGSO & AGSO.STCODES ******


def _code_strata():
    return ['code', 'label', 'name', 'pattern',
                  'size', 'density', 'thickness', 'color']


def agso_data (): 
    """
    Geological data codes processing
    
    .. deprecated:: function  will later deprecated 
    
    """
    
    agsofiles =['AGSO.csv', 'AGSO_STCODES.csv']
    
    for file in agsofiles : 
        path_to_agsofile=os.path.join(os.environ["pyCSAMT"], 'geodrill',
                                                 "_geocodes", file)
        with open(path_to_agsofile,'r', encoding='utf8') as f:
                fgeo=f.readlines()
        for ss, stratum in enumerate(fgeo):
            stratum=stratum.strip('\n').split(',')
            fgeo[ss]=stratum
        if file =='AGSO.csv':
            fagso=fgeo
        elif file =='AGSO_STCODES.csv':
            fagso_stcodes=fgeo
            fagso_stcodes[0]=fagso[0]
        else :
            _logger.warn("Geocodes files < AGS0 & AGS0_STCODES> Not found !")
            CSex.pyCSAMTError_file_handling('Geocodes files :< AGS0 & AGS0_STCODES> not found.'\
                                            'check the right path to _geocodes folder. ')
    return fagso, fagso_stcodes


def set_stratum_on_dict():
    """
    Process to put geocodes_strata and geocodes_structures into dictionnaries  
    better way to go on metaclasses merely. Thus each keys of dictionary will be 
    its own object. 

    Returns
    -------
        strata_dict : dict
            Disctionnary of geostrata 
        structures_dict : dict
            Dictionnary of geostructures.

    """
    code_strata=_code_strata()
    drop_bar=[' / ',' ', '/','-']
    attr_dict={}
    strata_dict, structures_dict={},{}
    for iiagso in range(2):
        fstrata=agso_data()[iiagso][1:]
        
        for ss , elem in enumerate(fstrata):
            for jj, attribute in enumerate(code_strata):
                attr_dict["{0}".format(attribute)]=elem[jj]
                stratum_description=elem[2]
            for db in drop_bar :
                if db in stratum_description : 
                    stratum_description=stratum_description.replace(db,'_')
            stratum_description=stratum_description.lower()
            if stratum_description.startswith('"') or stratum_description.endswith('"'):
                stratum_description=stratum_description[1:-1]
            if iiagso ==1 :
                structures_dict['{0}'.format(stratum_description)]=attr_dict
            else :
                strata_dict['{0}'.format(stratum_description)]=attr_dict
            attr_dict={}

    return strata_dict, structures_dict

#***  end call file from _geocodes folder :AGSO & AGSO.STCODES ******

#***start Avg files extract **** 
    
def _validate_avg_file(avg_dataLines):
    
    """
    Function to validate and readZonge Avg FILE :Plainty file _F1 and the Astatic file 
    _F2. Both files can be read and separatedd by the program. if Avg file doesnt match 
    any of them , the programm will generate errors . 
    
    Parameters 
    -----------
        * avag_dataLines: list 
                    list of avgData for all lines. 
    Returns 
    --------
        int , 
            type of AVG Zonge file , _F1 & _F2: 
  
                
    """
        
    remark =['\\ AMTAVG','$ ASPACE', '$ XMTR '] 
    count=0
    
    import inspect 

    func_name = inspect.getframeinfo(inspect.currentframe())[2]
    _logger.info ('Check the Avg_Datalinesfile %s',func_name )
      
    if avg_dataLines is not ['']:
        filehead="".join([ss for ss in avg_dataLines[:3]])
        #------------------checker section ------------------------
        #--->  check if astatic file is on the head_section 
        if filehead.find('astatic'.upper())>1: 
            _FT=2
        elif 'astatic'.upper() not in filehead: 
            for ss in remark :
                if filehead.find(ss) > 0 :count +=1
            #---> that mean all the remak elemt are on the head_section 
            if count >=2 :
                _FT=1
        else:
            _logger.error('Wrong Avg file.  Must respect the typical zonge file.')
            raise CSex.pyCSAMTError_avg_file('Type of AVg file provide is Wrong. No match'\
                                             'the typical Zonge engeneering file.')       
    else : 
        _logger.error('No Zonge Engeneering files exists. No data found on that curent AVG file.'\
                      'Check the better path to Avgfiles right file.')
        raise CSex.pyCSAMTError_avg_file('Check the The type of Avg file.')
            #-------------------end checker -----------------------------
    return  _FT
        



def straighten_cac2CSfile (data_array, component_column_section = None):
    """
    Sometimes head_sections of file _F2(CAC2CSAMT) provided is little 
    different colunms section name  according to different version . 
    it 's better to filter  and to check before returning the 
    correct informations we need.
    
    Parameters  
    ------------
        * data_array : ndarray 
                data from AVG astatic file 
       
        * component_column_section : list 
                astactic file column comps provided 
         
    Returns
    --------
        array_we_need :ndarray
                 same infos present in the plainty /1 Avg file 
        array_other_comp : pd.Core.DataFrame 
                 infos include through Astatic softwares 
                 very usefull therefore we keep it .
        
             
    """
    _comp_not_in_F1, _comp_we_need=['Z.mwgt','Z.pwgt','Z.mag',
                                     'SRes','E.wght',
                                     'H.wght','Z.%err',
                                     'Z.perr','Gdp.Blk',
                                     'Gdp.Chn', 'Gdp.Time'],['skp','Freq',
                                                            'Tx.Amp', 'E.mag',
                                                            'B.mag','B.phz',
                                                            'Z.phz','ARes.mag',
                                                            'E.%err','E.perr','B.%err',
                                                            'B.perr','Z.perr',
                                                            'ARes.%err','E.phz']           
                                            
    
    if component_column_section is None : 
        raise CSex.pyCSAMTError_inputarguments('head-section must be on list of string items.')
        
    tem_del, other_del=[],[]
    #strip comps _column to check....
    component_column_section=func._strip_item(item_to_clean=component_column_section)
    
    #---> put on dataFrame the component wee_need 
    df=pd.DataFrame(data=data_array, columns= component_column_section)
    for ss in component_column_section : 
        if ss not in _comp_we_need: 
            tem_del.append(ss)
        if ss in _comp_we_need :
            other_del.append(ss)
    df_we_need=df.drop(tem_del, axis=1)
    df_we_need.reset_index(drop=True, inplace=True)
    array_we_need= df_we_need.to_numpy()
    #--->> _keep on other dataframe the component we dont need.
    # other_comp_list =func._cross_eraser(data=component_column_section, to_del=tem_del,
    #                                     deep_cleaner=False)
    df_other_comp=df.drop(other_del, axis=1)
    df_other_comp.reset_index(drop=True, inplace=True) 

    
    #--> add skpi value if not in component section : 
    if 'skp'  not in [ii.lower() for ii in component_column_section]: # to spare a manner 'Skp' is written.
        skp_value =2   #  give a certitude value that data were good quality : 
        skp_array=np.full((array_we_need.shape[0],1),skp_value)
        array_we_need=np.concatenate((skp_array,array_we_need), axis=1)

    
    return array_we_need , df_other_comp


def truncated_data (data, number_of_reccurence, **kwargs):
    """
    Function to truncate all data according to number of frequency.
    
    Parameters
    ----------
        * data : list, or nd.array 
                data must be truncate.
        * number_of_freq : int
                number of frequency imaged.

    Returns
    -------
        list
            loc_list , data truncated on list.

    """
    if type(data) is list : 
        data =np.array(data) 
    value_lengh= data.shape[0]
    # index_loc = [np.int(ss) for ss in np.linspace(0, value_lengh, number_of_station)]
    index_loc = [ss for ss in np.arange(0,value_lengh, number_of_reccurence)]

    loc_list=[]
    for ss, value  in enumerate(index_loc) : 
        if ss ==len(index_loc)-1 : 
            loc_list.append(data[value:])
            break
        loc_list.append(data[value:index_loc[ss+1]])
            
    return loc_list 
    

def _numbering_station(number_of_station, number_of_freq): 
    """
    small function to numbering stations .
    
    Prameters 
    ---------
        * number_of_station : int  
            number of station found on the site. 
        * number_of freq : int  
            number of frequency found for survey at each station.
        
    Returns
    --------
        numbsta : str 
              station numbered 
        poly_sta : list 
             multplie station for each frequency for each stations.
    """
   
    numbsta =['S{:02}'.format(ii) for ii in range (number_of_station)]
    poly_sta=numbsta*number_of_freq 
    poly_sta.sort()
    
    return numbsta , poly_sta


def zstar_array_to_nan (zstar_array, nan_value=np.nan, keep_str =False): 
    """
    Parameters
    ----------
        * zstar_array : ndarray
                array contain unfloat converter value. the unconverter value can be a '*'
                
        * nan_value : float or np.nan type
                the nan_value could be any value either int, float or str. 
                The *default* is np.nan.
                
        * keep_str : bool, optional
                keep the str item on your array. f keep_str is set to 
                false and the type nan_value is str , the program will force 'keep_str_'  to True 
                to allow converter .
                The *default* is False.

    Returns
    -------
         ndarray 
            zstrar_array converted .

    """
    
    for kk , value in enumerate(zstar_array): 
        try : zstar_array[kk] =float(value)
        except :zstar_array[kk]=nan_value
    if type(nan_value) is str : keep_str=True
    if keep_str : return zstar_array 
    return np.array([float(kk) for kk in zstar_array])

   
def get_array_from_reffreq ( array_loc, freq_array,reffreq_value, stnNames=None):
    """ 
    Get array value at special frequency
    Parameters
    ------------
        * array_loc : dict , 
            dictionnary of stations , array_value e.g: S00:(ndarray,1) rho_values
            
        * freq_array : (ndarray,1) 
            frequency array for CSAMT survey 
            
        * reffreq_value : int or float 
            the value of frequency user want to get the value 
            
        * stnNames : list 
            list of stations names . 
    
    Returns 
    ---------
        array_like
            an array of all station with reffreq_value . 
            e.g reffreq_value =1024. it return all value of the array at 1024Hz frequency . 
    
    """
    if stnNames is None : raise CSex.pyCSAMTError_station('You may at least specify '\
                        'the array or list of stations-Names or station id.')
    
    arrayObj,freqObj,rfObj,stnObj =array_loc ,freq_array,reffreq_value,stnNames

    def get_reffreq_index(freq_array, reffreq_value): 
        """ 
        get the index of reference index. From this index ,All array will filter data at this reffreq
        value . 
        
        """
    
        for ii, freq in enumerate(freq_array): 
            if freq == reffreq_value:
                index_rf = ii
                break
        return index_rf
    
    return np.array([values[get_reffreq_index(freq_array=freqObj, reffreq_value=rfObj)] \
                for stn in stnObj for keys, values in arrayObj.items() if stn==keys])
        
def relocate_on_dict_arrays(data_array, number_of_frequency, station_names =None): 
    """ 
    Put data arrays on dictionnary where keys is each station and value the array of that station.
    if station_names is None , program will create name of station. if station_names is given ,
    function will sorted stations names . please make sure to provide correctly station according 
    the disposal you want . 
    
    :param number_of_frequency: array of frequency during survey 
    :type number_of_frequency: array_like
    
    :param station_names:  list of station 
    :type station_names: list of array_like 
        
    :returns: infos at data stations 
    :rtype: dict 
    
    """
        
    if station_names is not None : 
        if type(station_names) is list :
            assert len(station_names) == (data_array.size / number_of_frequency),\
                CSex.pyCSAMTError_station('Number of Station provided must be <{0}> not <{1}>'.format(np.int(data_array.size / number_of_frequency)), 
                                            len(station_names))
        else :
            assert station_names.size == data_array.size / number_of_frequency, \
            CSex.pyCSAMTError_station('Number of Station provided must be <{0}> not <{1}>'.format(np.int(data_array.size / number_of_frequency)), 
                                            station_names.size)
            station_names =station_names.tolist()
            
        NUM_STN=sorted(station_names)
    elif station_names is None  :
        station_length =np.int(data_array.size / number_of_frequency )
        
        NUM_STN=sorted((_numbering_station(number_of_station=station_length ,
                                    number_of_freq=number_of_frequency))[0])

    LIST_BROK = truncated_data(data=data_array, number_of_reccurence=number_of_frequency)   
    
    return { key: value for key, value in zip (NUM_STN, LIST_BROK)}

def dipole_center_position (dipole_position=None):
    """
    Generaly positions are taken at each electrode of dipole to that to easy correct data  for ploting and for
    noise correction , we adjust coordinate by taking the center position that means , 
    the number of points will be substract to one.
    
    :param dipole_position: postion array at each electrodes. 
    :type dipole_position: array_like 
    
    :returns: centered position value  array 
    :rtype: array_like 
    
    """
    if dipole_position is None : raise CSex.pyCSAMTError_inputarguments('NoneType can not be compute.'\
                                                                        ' Please provide the right values.')
    try : dipole_position= np.array([float(pp) for pp in dipole_position])
    except : raise CSex.pyCSAMTError_float('Cannot convert position values. '\
                                           'Position must be a number not str.Please check your data!.')
    # temp_pospp =[(dipole_position[pp]+dipole_position[pp-1])/2 for \
    #              pp, pos in enumerate( dipole_position) if ( pp>0 and pp <= dipole_position.size) ] 
    return np.array([(dipole_position[pp]+dipole_position[pp-1])/2 for \
                 pp, pos in enumerate( dipole_position) if ( pp>0 and pp <= dipole_position.size) ])
        

def get_matrix_from_dict(dict_array, flip_freq =False , 
                         freq_array= None , transpose =False ):
    """
    Function to collect dictionnary  station data with values as array 
    and build matrix along the line, The rowlines is assumed to be frequency 
    and colums as stations names. If `freq_array` is provided , `flip_freq` is
    not usefull.
    
    Parameters
    ----------
    dict_array : dict
        stations dictionnary array, keys are stations names and values are 
        stations values on array_like.
    transpose : bool, optional
        transpose  matrix , if true wwill transpose data and 
        stations should be read as rowlines and frequency as columns.
        The default is False.
    freq_array : array , optional 
        array of frequency, if provided will check if frequency are range from 
        highest to lowest .Defalut is True
    flip_freq :bool, optional 
        set to false if you want frequency to be lowest to highest otherwise
         frequency are sorted Highest to lowest. Default is False

    Returns
    -------
    ndarray(len(stations,), len(frequency))
        matrix of dictionnary values 
    bool, 
        flip_freq, ascertain if frequency array is flipped or not. 
    """
    
    if not isinstance(dict_array, dict): 
        raise CSex.pyCSAMTError_processing('Main argument `dict array`'
                                           ' should be dictionnary'
                                           ', not {0}'.type(dict_array))
        
    if freq_array is not None :
        if isinstance(freq_array, (list, tuple)):
            freq_array =np.array(freq_array)
            try :
                freq_array=freq_array.astype('float')
            except :
                raise CSex.pyCSAMTError_float('Could not convert array to value !')
        # now check the order 
        if freq_array[0] < freq_array[-1]: 
            freq_array = freq_array[::-1]
            flip_freq = True        # set flip to True to flip data values  
            
    tem= [values for stn , values in dict_array.items()]
    matrix = func.concat_array_from_list(tem, concat_axis=1)
    
    if flip_freq is True : matrix =matrix [::-1]    # reverse matrix 
    if transpose is True : matrix =matrix.T         # Transpose matrix 
    
    return matrix, flip_freq
        

def plot_reference_frequency (reference_array, frequency_array, 
                              data_array,
                              station_names =None ,  **kwargs):
    """
    Function to plot reference frequency 
    
    Parameters
    ----------
    reference_array : array_like, 
        array_of average_impedance Z_avg.
    frequency_array: array_like 
        array of frequency on sites 
    data_array : ndarray,
        nadarray of sounding curve, ndarray(len(frequency), len(number_ofstaions).

    Returns
    -------
    viewer
    """
    import matplotlib.pyplot as plt 
    
    ls =kwargs.pop('ls', '-')
    marker =kwargs.pop('marker', 'o')
    ms=kwargs.pop('ms', 0.2)
    lw =kwargs.pop('lw', 2)
    fs =kwargs.pop('fs', 0.8)
    if isinstance(data_array, dict): 
        data_array =sorted(data_array)  # sorted ditionnaries 
        station_names = list(data_array.keys())
        data_array = func.concat_array_from_list(list(data_array.values(), 
                                                      concat_axis=1))
    
    if station_names is None : 
        station_names =['S{0:02}'.format(ii) 
                        for ii in range(data_array.shape[1])]
    
    fig =plt.figure()
    ax =fig.add_subplot(111)
    
    ax.loglog(frequency_array,
                reference_array, 
                c='r', ls =ls, lw=lw, ms=ms *fs , 
                marker =marker)
    for ii in range(data_array.shape[1]):
        
        ax.loglog(frequency_array, data_array[:, ii])
        
    plt.show()
    
    
    
    

# if __name__=='__main__':
    
#     # print(numfreq, num_sta)
#     ch =['2', '7', '9', 'a', 'g', 'e', '*', '9','2', '7', '9', '*', 'g', 'e', 'geth', '9']
#     array =np.array(ch)
#     test=zstar_array_to_nan(zstar_array=array, keep_str=False)

    
    
    
    
    
    
    
    
    
    


# -*- coding: utf-8 -*-
"""
    .Console scripts. 
    $ python pycsamt write_drillhole 
    
    write drill hole from location drill parser file . To see how parser file is 
    organized , consult ~/ 'drill_example_files' directory.
    exemple of parser file is 'nbleDH.csv'
    to write drillhole individually either `Collar`,`Geology`,
    `Sample`,`Elevation`, or `Azimuth` and to also add elevation or azimuth
    please use the scripts in ~/scripts directory. 
    
    default`*` means write all data. 
    
Created on Tue Apr  6 21:33:23 2021

@author: @Daniel03

"""
import os 
from pycsamt.geodrill.geoCore.geodrill import Drill 
from pycsamt.gui.wrap_console_scripts import wrap_cscripts as wrs

default_kwargs = {'Path to parser drill files - str - ' : None , 
                  'Name of parser file  - str - '  : False, 
                  'Type of data to write - str - ':'*',
                  'Save borehole outdir  - str - ':None,
                  }
      
def main(): 
    
    wrs_obj = wrs(default_kwargs =default_kwargs)
    for kwey, kwvalues in wrs_obj.sanitize_kwargs.items():
        if kwey.find('Path to parser drill') >=0 : path_to_parser_files = kwvalues
        if kwey.find('Name of parser file ')>=0 : parser_file = kwvalues
        if kwey.find('Type of data to write') >= 0 : kind_of_data2output = kwvalues 
        if kwey.find('Save borehole outdir') >= 0 : savepath = kwvalues 
 

    borehole_obj = Drill (well_filename= os.path.join(path_to_parser_files, parser_file)  , 
                       auto= True)
    borehole_obj._collar()
    borehole_obj.dhGeology()
    borehole_obj.dhSample()
    borehole_obj.dhSurveyElevAz(add_azimuth=None, 
                                add_elevation=None)
    borehole_obj.writeDHData(data2write=kind_of_data2output, 
                             savepath  = savepath )
if __name__ == '__main__': 
    main()



# define mtpy release version below
# see https://packaging.python.org/guides/single-sourcing-package-version/ 
__version__ = "1.0.9"

# load pycsamt default logging config
import sys 
import os 
import logging 

from pycsamt.utils._csamtpylog import csamtpylog

if __package__ is None or __name__ == '__main__': 
    sys.path.append( os.path.dirname(os.path.dirname(__file__)))
    sys.path.insert(0, os.path.dirname(__file__))
    __package__= 'pycsamt'

csamtpylog.load_configure(os.path.join(os.path.abspath('.'),
                                        'pycsamt', '_logfile',
                                        "main_logging_configfile.yml"))

# set loging Level
logging.getLogger('matplotlib').setLevel(logging.WARNING)




# -*- coding: utf-8 -*-
"""
===============================================================================
    Copyright © 2021  Kouadio K.Laurent
    
    This file is part of pyCSAMT.
    
    pyCSAMT is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    
    pyCSAMT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public License
    along with pyCSAMT.  If not, see <https://www.gnu.org/licenses/>.

===============================================================================  
.. _module-Structural::`pycsamt.geodrill.geoCore.structural` 
    
    :synopsis:class for  geological structural analysis 
               contains some conventional structure can populate the data
    
Created on Sat Nov 28 21:19:13 2020

@author: @Daniel03

"""


import os, warnings
import numpy as np 
from pycsamt.ff.processing.callffunc import set_stratum_on_dict as strato
from pycsamt.utils  import exceptions as CSex
from pycsamt.utils._csamtpylog import csamtpylog
_logger=csamtpylog.get_csamtpy_logger(__name__)

#---- End import modules --------

class geo_pattern: 
    """
    Singleton class to deal with geopattern  with other modules. It is and exhaustive pattern
    dict, can be add and change. This pattern  will be depreacted later , to create for pyCSAMT,
    its owwn geological pattern in coformity with the conventional geological swatches .
     deal with USGS(US Geological Survey ) swatches- references and FGDC (Digital cartographic 
    Standard for Geological  Map Symbolisation -FGDCgeostdTM11A2_A-37-01cs2.eps)
         
    make _pattern:{'/', '\', '|', '-', '+', 'x', 'o', 'O', '.', '*'}
            /   - diagonal hatching
            \   - back diagonal
            |   - vertical
            -   - horizontal
            +   - crossed
            x   - crossed diagonal
            o   - small circle
            O   - large circle
            .   - dots
            *   - stars
    """
    pattern={
                        "basement rocks" :      ['.+++++.', (.25, .5, .5)],
                        "igneous rocks":        ['.o.o.', (1., 1., 1.)], 
                        "duricrust"   :         ['+.+',(1., .2, .36)],
                        "gravel" :              ['oO',(.75,.86,.12)],
                        "sand":                 ['....',(.23, .36, .45)],
                        "conglomerate"    :     ['.O.', (.55, 0., .36)],
                        "dolomite" :            ['.-.', (0., .75, .23)],
                        "limestone" :           ['//.',(.52, .23, .125)],
                       "permafrost"  :          ['o.', (.2, .26, .75)],
                        "metamorphic rocks" :   ['*o.', (.2, .2, .3)],
                        "tills"  :              ['-.', (.7, .6, .9)],
                        "standstone ":          ['..', (.5, .6, .9)],
                        "lignite coal":         ['+/.',(.5, .5, .4)],
                        "coal":                 ['*.', (.8, .9, 0.)],
                        "shale"   :             ['=', (0., 0., 0.7)],
                        "clay"   :              ['=.',(.9, .8, 0.8)],
                        "saprolite" :           ['*/',(.3, 1.2, .4)],
                        "sedimentary rocks":    ['...',(.25, 0., .25)],
                        "fresh water"  :        ['.-.',(0., 1.,.2)],
                        "salt water"   :        ['o.-',(.2, 1., .2)],
                        "massive sulphide" :     ['.+O',(1.,.5, .5 )],
                        "sea water"     :       ['.--',(.0, 1., 0.)],
                        "ore minerals"  :       ['--|',(.8, .2, .2)],
                        "graphite"    :         ['.++.',(.2, .7, .7)],
                        }
            
class Structure :
    """
    Class for typical geological strutural conventions 
    for AGSO_STCODES .  All geological structural informations are
    geostructral object.
                  
    Holds the following information:
        
    ==========================  ===============  ===================================
    Attributes                  Type             Explanation
    ==========================  ===============  ===================================
    boudin_axis                 geos_obj         boudin    
    fold_axial_plane            geos_obj         axial plam of structural fold.
    banding_gneissosity         geos_obj         gneissossity of boudin plan  
    s_fabric                    geos_obj         fabric plan
    fault_plane                 geos_obj         fault plan 
    fracture_joint_set          geos_obj         fracture joint 
    undifferentiated_plane      geos_obj         unnamed geological structure 
    sharp_contact               geos_obj         sharp contact `various discrepancy` 
                                                 contact `stratigraphy discrepancy`  
                                                 fracture or fault discrepancies
    ==========================  ===============  ====================================

    More attributes can be added by inputing a key word dictionary

    :Example: 
        
        >>> from pycsamt.geodrill.geoCore.structural import Structure
        >>> structural=Structure()
        >>> boudin=boudin_axis()
        >>> print(boudin.code) 
        >>> print(structural.boudin_axis.name)
        >>> print(structural.boudin_axis.color)
    """  
    _logger.info('Set Structural main geological informations. ')
    
    def __init__(self, **kwargs):
        
        
        self.boudin_axis =boudin_axis()
        self.fold_axial_plane=fold_axial_plane()
        self.banding_gneissosity=banding_gneissosity()
        self.s_fabric=s_fabric()
        self.fault_plane=fault_plane()
        self.fracture_joint_set=fracture_joint_set()
        self.undifferentiated_plane=undifferentiated_plane()
        self.sharp_contact=sharp_contact()
        
        
        for keys in list(kwargs.keys()):
            setattr(self, keys, kwargs[keys])

#====================================================
#   geological structural objects 
#=====================================================
class boudin_axis(object):
    """
    Special class for boudins_axis
    
    Holds the following information:
    
    =================  =============  ========================================
    Attributes         Type             Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    =================  =============  ========================================

    More attributes can be added by inputing a key word dictionary
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
        self._set_boudin_axis()   
        
    def _set_boudin_axis(self):
        """
        methode to populates attributes

        """
        baxis=strato()[1]
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')

class fold_axial_plane(object):
    """
    Special class for fold_axial_plane
    
    Holds the following information:

    =================  =============  ========================================
    Attributes              Type          Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    =================  =============  ========================================

    More attributes can be added by inputing a key word dictionary
    
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        self._set_fold_axial_plane()
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
    def _set_fold_axial_plane(self):
        """
        Methode to populates attributes.

        """
        baxis=strato()[1]
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')

class banding_gneissosity(object):
    """
    Special class for banding_gneissosity
    
    Holds the following information:

    =================  =============  ========================================
    Attributes              Type          Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    =================  =============  ========================================

    More attributes can be added by inputing a key word dictionary
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        self._set_banding_gneissosity()
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
    def _set_banding_gneissosity(self):
        """
        methode to populates attributes

        """
        baxis=strato()[1]
        # print(baxis)
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')             

class s_fabric(object):
    """
    Special class for s_fabric
    
    Holds the following information:

    =================  =============  ========================================
    Attributes              Type          Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    =================  =============  ========================================

    More attributes can be added by inputing a key word dictionary
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        self._set_s_fabric()
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
    def _set_s_fabric(self):
        """methode to populates attributes"""
        baxis=strato()[1]
        # print(baxis)
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')  

class fault_plane(object):
    """
    Special class for fault_plane
    
    Holds the following information:

    =================  =============  ========================================
    Attributes          Type            Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    =================  =============  ========================================

    More attributes can be added by inputing a key word dictionary
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        self._set_fault_plane()
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
    def _set_fault_plane(self):
        """
        methode to populates attributes

        """
        baxis=strato()[1]
        # print(baxis)
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')
                
class fracture_joint_set(object):
    """
    Special class for fracture_joint_set
    
    Holds the following information:

    =================  =============  ========================================
    Attributes         Type           Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    =================  =============  ========================================

    More attributes can be added by inputing a key word dictionary
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        self._set_fracture_joint_set()
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
    def _set_fracture_joint_set(self):
        """methode to populates attributes
        """
        baxis=strato()[1]
        # print(baxis)
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')
                
class undifferentiated_plane(object):
    """
    Special class for undifferentiated_plane
    
    Holds the following information:

    =================  =============  ========================================
    Attributes         Type             Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    =================  =============  ========================================

    More attributes can be added by inputing a key word dictionary
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        self._set_undifferentiated_plane()
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
    def _set_undifferentiated_plane(self):
        """
        methode to populates attributes

        """
        baxis=strato()[1]
        # print(baxis)
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')             
                
class sharp_contact(object):
    """
    Special class for sharp_contact
    
    Holds the following information:

    =================  =============  ========================================
    Attributes          Type           Explanation
    =================  =============  ========================================
    code                str             conventional code    
    label               str             named label
    size                str             drawing size  
    pattern             str             drawing pattern 
    density             str             elmts density 
    thickness           str             drawing thickness 
    color               str             color set
    ==================  =============  ========================================
    
    More attributes can be added by inputing a key word dictionary
    """
    def __init__(self, **kwargs):
        
        self.code =None 
        self.label=None 
        self.name=None 
        self.pattern=None 
        self.size =None 
        self.density=None 
        self.thickness=None
        self.color =None
        
        self._set_sharp_contact()
        
        for keys in list(kwargs.keys()):
            self.__setattr__(keys, kwargs[keys])
            
    def _set_sharp_contact(self):
        """
        methode to populates attributes
        """
        baxis=strato()[1]
        # print(baxis)
        for keys, values in baxis.items():
            if keys == self.__class__.__name__:
                self.code =values.__getitem__('code')
                self.label=values.__getitem__('label')
                self.name=values.__getitem__('name')
                self.pattern=values.__getitem__('pattern')
                self.size=values.__getitem__('size')
                self.color=values.__getitem__('color')
                self.density=values.__getitem__('density')
                self.thickess=values.__getitem__('thickness')             
                
class Geo_formation (object): 
    """
    This class is an axilliary class to supplement geodatabase , 
    if the GeodataBase doesnt reply  to SQL request  , then use this class
    to secah information about structures .  If SQL is done as well ,
    program won't call this class as rescure . 
    Containers of more than  150 geological strutures.
        
   
    
    .. note:: replace in attributes param "**" by  the *name of struture*
    
    ==================  ============  =========================================
    Attributes          Type           Explanation
    ==================  ============  =========================================
    names               array_like      names of all geological strutures 
    codes               array_like      names of all geological codes 
    **code              str             code of specific geological structure 
    **label             str             label of specific structure
    **name              str             label of specific structure
    **pattern           str             pattern of specific structure  
    **pat_size          str             pattern size  of specific structure
    **pat_density       str             pattern density l of specific structure
    **pat_thickness     str             pttern thickess of specific structure
    **color             str             color of specific structure
    ==================  ============  =========================================

    1.  To see the names of strutures , write the script below 
    
    :Example:
        
        >>> from pycsamt.geodrill.geoCore.strutural import Geo_formation as gf 
        >>> geo_structure = gf()
        >>> geo_structure.names
        
    2.  To extract color  and to get the code of structure  like amphibolite 
    
    :Example:
        
        >>> from pycsamt.geodrill.geoCore.strutural import Geo_formation as gf 
        >>> geo_structure = gf() 
        >>> geo_structure.amphibolite['color'] 
        >>> geo_structure.amphibolite['code'] 
        >>> geoformation_obj.AMP['color']
        ... 'R128G128'
    """ 
    codef =['code','label','__description','pattern', 'pat_size',	'pat_density',
            'pat_thickness','color']

    def __init__(self, agso_file =None , **kwargs) :
        self._logging = csamtpylog.get_csamtpy_logger(self.__class__.__name__)
        
        self._agso_fn =agso_file
        
        for key in list(kwargs.keys()): 
            self.__setattr__(key, kwargs[key])
            
        if self.agso_fn is not None : 
            self._set_geo_formation()

    @property 
    def agso_fn (self): 
        if self._agso_fn is None : 
             self._agso_fn = os.path.join(os.path.abspath('.'),'pycsamt', 'geodrill', 
                                         '_geocodes','agso'.upper() + '.csv' )
        if self._agso_fn is not None : 
            if os.path.isfile(self._agso_fn) : 
                with open(self._agso_fn, 'r', encoding ='utf8') as _fags0: 
                    agso_lines = _fags0.readlines()
                    _secags0 = agso_lines[0].strip().split(',')
                    _secags0 =[_code.lower() for _code in _secags0]
                    self._AGS0_DATA =agso_lines[1:]
                    
                    if _secags0 != self.codef :
                        warnings.warn(' !Problem to decode geostructures structures!')
                        raise CSex.pyCSAMTError_structural('Geostructures files provided is wrong !')
            else : 
                
                warnings.warn('No Geostructure file detected .{_structures unfound}. It seems the file moved from its host folder.')
                raise CSex.pyCSAMTError_structural('pyCSAMT inner geocodes property is not in host folder {_geocodes_}.'\
                                                   ' Please provide a right structures codes.')
            return self._AGS0_DATA 
    
    def _set_geo_formation(self, agso_fn =None): 
        """
        Read and set  attributes and decode agso geostructures .
        
        :param agso_fn:  full path geological structural  file.
        :type agso_fn: str 
        
        :Example: 
            
            >>> geoformation_obj =Geo_formation()
            >>> DATA = geoformation_obj._AGS0_DATA
            >>> print( geoformation_obj.argillite['name'])
            >>> print( geoformation_obj.argillite['color'])
            >>> print( geoformation_obj.wood['color'])
            >>> print( geoformation_obj.amphibolite['code'])
            >>> print( geoformation_obj.names)
        """
        self._logging.info('Read &  and decodes geostructures files . ')
        if agso_fn is not None : 
            self.agso_fn= agso_fn 
            
        #split the code files 
        
        self._AGS0_DATA = [ geos.strip().split(',') for ii, geos in enumerate(self._AGS0_DATA)]
        self._AGS0_DATA= np.array (self._AGS0_DATA)

        # sanitize the geonames . 
        self.__setattr__('names',
                         np.array( [name.replace('"', '') for name in self._AGS0_DATA[:, 2]] ))
        
        self.__setattr__('codes', self._AGS0_DATA[:, 0])
        
        # set for all values in geofomations codes 
        for ii, codeff in enumerate(self.codef [1:],1) :  # count starting from one .
            if codeff.find('pat') >=0 : 
                for jj, pp in enumerate(self._AGS0_DATA[:, ii]): 
                    if pp == '' or pp ==' ' :       # repalec all None value by 0. later will filled it 
                        self._AGS0_DATA[:, ii][jj]=0.
                        
                self.__setattr__(codeff, 
                                 np.array([float(pp) for pp in self._AGS0_DATA[:, ii]]))
            else : 
               self.__setattr__(codeff,
                         np.array( [name.replace('"', '') for name in self._AGS0_DATA[:, ii]] )) 
        
        DECODE ={}
        # change RGBA color palette into Matplotlib color 
        self.mpl_colorsp= [ get_color_palette(rgb) for rgb in self._AGS0_DATA[:,-1]]

        for jj,  decode  in enumerate(self._AGS0_DATA):
           for ii, codec  in enumerate( self.codef) : 
               if codec =='__description' : codec = 'name'
               if codec =='color': 
                   DECODE[codec]= self.mpl_colorsp[jj]
               else : DECODE [codec]= decode[ii]
               self.__setattr__(decode[2].strip().replace('"', '').replace(' ', '_'),
                                DECODE)
               self.__setattr__(decode[0].strip().replace('"', '').replace(' ', '_'),
                                DECODE)
               
           DECODE ={}  
        
        
def get_color_palette (RGB_color_palette): 
    """
    Convert RGB color into matplotlib color palette. In the RGB color system two bits
    of data are used for each color, red, green, and blue. That means that each color runs
    on a scale from 0 to 255. Black would be 00,00,00, while white would be 255,255,255.
    Matplotlib has lots of pre-defined colormaps for us . They are all normalized to 255,
    so they run from 0 to 1. So you need only normalize data, then we can manually 
    select colors from a color map  

    :param RGB_color_palette: str value of RGB value 
    :type RGB_color_palette: str 
        
    :returns: rgba, tuple of (R, G, B)
    :rtype: tuple
     
    :Example: 
        
        >>> from pycsamt.geodrill.geoCore.structural import get_color_palette 
        >>> get_color_palette (RGB_color_palette ='R128B128')
    """   
    def ascertain_cp (cp): 
        if cp >255. : 
            warnings.warn(' !RGB value is range 0 to 255 pixels , '
                          'not beyond !. Your input values is = {0}.'.format(cp))
            raise CSex.pyCSAMTError_parameter_number('Error color RGBA value ! '
                                                     'RGB value  provided is = {0}.'
                                                     ' It is larger than 255 pixels.'.format(cp))
        return cp
    if isinstance(RGB_color_palette,(float, int, str)): 
        try : 
            float(RGB_color_palette)
        except : 
              RGB_color_palette= RGB_color_palette.lower()
             
        else : return ascertain_cp(float(RGB_color_palette))/255.
    
    rgba = np.zeros((3,))
    
    if 'r' in RGB_color_palette : 
        knae = RGB_color_palette .replace('r', '').replace('g', '/').replace('b', '/').split('/')
        try :
            _knae = ascertain_cp(float(knae[0]))
        except : 
            rgba[0]=1.
        else : rgba [0] = _knae /255.
        
    if 'g' in RGB_color_palette : 
        knae = RGB_color_palette .replace('g', '/').replace('b', '/').split('/')
        try : 
            _knae =ascertain_cp(float(knae[1]))
        except : 
            rgba [1]=1.
            
        else :rgba[1]= _knae /255.
    if 'b' in RGB_color_palette : 
        knae = knae = RGB_color_palette .replace('g', '/').split('/')
        try : 
            _knae =ascertain_cp(float(knae[1]))
        except :
            rgba[2]=1.
        else :rgba[2]= _knae /255.
        

    return tuple(rgba)
    

    
                
if __name__=="__main__":
    
    # structural=Structure()
    # boudin=boudin_axis()
    # # print(boudin.code) 
    # print(structural.boudin_axis.name)
    # print(structural.boudin_axis.color)
    
    geoformation_obj =Geo_formation()
    DATA = geoformation_obj._AGS0_DATA
    rgb=  geoformation_obj.amphibolite['color']
    print(rgb)


    
                
    
    


# -*- coding: utf-8 -*-
"""
Created on Fri Sep  4 15:49:10 2020

@author: Administrator

"""
import os 
import glob
import shutil
import pycsamt.ff.deprecatedmodules.pyconvert as mconv

#=============================


class Gui_Ediwrite:
    """Create Graphical User Interface from Tkinter """
    
    def __init__(self,edipath, 
                 bln_coordinates_filepath, coordinateFile,savepath):

        self.edipath=edipath
        self.blnpath=bln_coordinates_filepath
        self.coordinateFile=coordinateFile
        self.savepath=savepath
        
        if self.edipath is None :
            raise FileExistsError
        
        if os.path.exists(self.blnpath):
            self.path_to_blnfile=os.path.join(self.blnpath,self.coordinateFile)
            
        if self.savepath is None :
            self.savepath=os.getcwd()
            
        self.rewrite_edifiles()
            
    def rewrite_edifiles(self):
        "methode to rewrite edifiles"
        os.chdir(self.edipath)
        edir=mconv.EdiRewrite(Path_to_edifiles=self.edipath,
            coordinates_bln_file=self.path_to_blnfile)
        edir.check_edi()
        edir.ex_DMS()
        edir.ediRewrite()
        self.info=edir.print_stats()
        # movefile 
        try : 
            os.mkdir('new_EDI')    
        except OSError as e :
            print(os.strerror(e.errno))
        self.savepath=self.edipath+'/new_EDI'
        
        slst=[file for file in os.listdir(self.edipath) if (file[0]=='S'and file[-4:]==".edi")]
        for file in slst: 
            shutil.move(file,self.savepath)

                
class Gui_AvgToJformat:
    """create graphical user from Thinker """
    
    def __init__(self,avgpath,profileFormat, 
                 mode,rotate_angle=180,azimuth=True ):
        self.dir_=avgpath 
        self.type=profileFormat
        self.azimut=azimuth
        self.mode=mode 
        self.rotate_angle=rotate_angle
        
        if self.dir_==None :
            raise FileNotFoundError
        
        self._writeToJformat()
        
    def _writeToJformat(self):
        '''write to jformat file'''
        os.chdir(self.dir_)
        filenames=glob.glob("*.AVG")  # reprend tous les fichiers *avg 
        # profile type files :
        # type1,type2='*.csv','*.txt'
        #change the type of file : 
        ptypefile=glob.glob(self.type)
        comp=0
        #change azimuth to False if azimuth=0 else Default is True   
        # azimut=False 
        
        #mode of survey : Transverse =TM=RYX 
        # mode ='RXY'
        
        for avgfile in filenames : 
        
            if filenames !='':
                fold="{}".format(avgfile[:-4])
                
            os.mkdir(self.dir_+'/{}'.format(fold))# creer un nouveau dossier .
                 
            path_pfile=ptypefile[comp]
            comp+=1
        
            
            # mode ='RXY'
            
            
            # main executor program ---
            ex_f =mconv.AvgToStations(avg_path=avgfile,
                                     mode=self.mode,
                                     rotate_angle=self.rotate_angle)
            ex_f.rawAvgToArray()
            ex_f.arrayToDf()
            ex_f.shrunk_df()
            ex_f.jchainf()
            pf_= mconv.Profile(path_pfile,azimuth=self.azimut)
            pf_.ex_profile()
            pf_.azimuth()
            pf_.build_jfile()
            ex_f.print_()
            
            save_path=os.path.join(self.dir_,'{}/'.format(fold) )
            
            # save_path=r'F:/OneDrive/Python/CodesExercices/' +\
            #             'ex_avgfiles/data/data_avg_control2/{}/'.format(fold)
            savefiles=glob.glob('*.dat')
            for file in savefiles :
        
                shutil.move(file,save_path)
                    
            

class Gui_StnToDat():
    """ Generate graphical user Interface from Tkinter """

    def __init__(self,stnpath=None, normalisation=False ,
                 utm_X_cor=300238.702, utm_Y_cor=2369.252 ):
        
        self.dir_=stnpath
        self.normalised=normalisation
        self.utm_X_cor=utm_X_cor
        self.utm_Y_cor=utm_Y_cor
        
        if self.utm_X_cor==None :
            self.utm_X_cor=0.
        if self.utm_Y_cor is None :
            self.utm_Y_cor=0.
            
        if self.dir_ is None :
            self.dir_=os.getcwd()
        else :
            raise FileNotFoundError
            
        self._rewrite_coordinate_file()
        
    def _rewrite_coordinate_file(self):
  
        os.chdir(self.dir_)
        
        try : 
            os.mkdir('stnToDat_exportfiles')    
        except OSError as e :
            print(os.strerror(e.errno))
        savepath=self.dir_+'/stnToDat_exportfiles'
        filenames=glob.glob('*.stn')
        for stn_file in filenames :
            #Fonctions d'arborescence    
            inst=mconv.StnToDat(filename=stn_file,
                                normalized=self.normalised,
                                utmVar_X=self.utm_X_cor,
                                utmVar_Y=self.utm_Y_cor)
            inst.convertToDat()
        datfile=glob.glob('*.dat')
        for file in datfile: 
            shutil.move(file,savepath)
        

class Gui_ITER2OASIS:
    """Generate GUI from Tkinter """
    
    def __init__(self, path_to_stn,file_format,
                 number_of_stations=1,step_btw_station=50, 
                 file_type=False,choix=False,savefig=False):
        self.dir_=path_to_stn
        self.number_of_stations=number_of_stations
        self.step_btw_station=step_btw_station
        self.file_format=file_format
        self.choix=choix
        self.file_type=file_type
        self.savefig=savefig
        
        self._write_oasfile()
        
    def _write_oasfile(self):
    
        os.chdir(self.dir_)
        
        #-------Main Program---------
        
        iter_filenames=glob.glob('*iter.dat')
        topo_cor_filenames=glob.glob("*_cor.dat")
        bln_filenames=glob.glob("*.bln")
        
        for file in iter_filenames :
            name=file[:-9]
            topofile=name+"_cor.dat"
            if topofile in topo_cor_filenames:
                blnfile=name+".bln"
                if blnfile in bln_filenames: 
                    # Fonction d(arborescence :)
                    exp_=mconv.iter2dToOasis(iterdatfiles=file, 
                              stndatfiles=topofile, 
                              blnfiles=blnfile, 
                              File_format=self.fileformat, step= self.step_btw_station, 
                              NumberOfStation=self.number_of_stations, 
                              FileType=self.file_type,
                              savefig=self.savefig)
                    exp_.arrangeIter2d()
                    exp_.extra_offDepthRho()
                    exp_.reoder()
                    exp_.useChoice(choice=self.choix)
            # Move to savepath
            #choice==True : Dont need to run this code:
        if self.choix==False :
            try : 
                os.mkdir('oasis_exportfiles')    
            except OSError as e :
                print(os.strerror(e.errno))
            savepath=self.dir_+'/oasis_exportfiles'
            ch=['*oas.csv','*oas.xlsx']
            for extend in ch :  
                datfile=glob.glob('*oas.csv')
                for file in datfile: 
                    shutil.move(file,savepath)
        #----End program-----

    
# if __name__=="__main__":
    
#     from tkinter import *
#     fen=Tk()
#     fen.title('AVG_App')
#     fen.geometry("500x50")
#     frame=Frame(fen)
#     frame.grid(row =4, column =4)
#     Button(frame, text="EDIREWRITE",relief="groove", command=Gui_Ediwrite).grid(row=1, column=1)
#     Button(frame, text='AVG-->JFORMAT', relief="groove",command=Gui_AvgToJformat).grid(row=1,column=2)
#     Button(frame, text="EXIT", command=fen.quit).grid(row=2, column=1,sticky=W)
#     fen.mainloop()
#     fen.destroy()
#     # ff.mainloop()

    


# -*- coding: utf-8 -*-
"""
Created on Mon Sep  7 12:01:17 2020

@author: Administrator
"""
import os 
from tkinter import *
from gui_avgapp import Gui_Ediwrite, Gui_StnToDat,Gui_AvgToJformat
from gui_edirewrite import Press_EDIR,Press_STND, Press_AVG, Press_OAS

#=============================

class MenuBar_EDIR(Frame):
    
    def __init__(self, boss=None):
        Frame.__init__(self, borderwidth=2,relief=GROOVE )
        #scrollingMenu Bar 
        fileMenu=Menubutton(self,text="EDIREWRITE")
        # fileMenu.pack(side=LEFT)
        fileMenu.grid(row=0, column=0, padx=20, pady=7, sticky=W )
        ##### Menu EDIREWRIE######
        me1=Menu(fileMenu)
        me1.add_command(label="Open", underline=0,
                        command=Press_EDIR)
        # integration of file Menu 
        fileMenu.configure(menu=me1)
        
        
        #### add AVG-->JFORMAT to Menu Bar ######
        self.avg_jformat=Menubutton(self, text="AVG-->JFORMAT")
        # self.avg_jformat.pack(side=LEFT, padx=10)
        self.avg_jformat.grid(row=0, column=1, padx=20,pady=7, sticky=W )
        # scrolling of Menu<AVG-->JFORMAT> 
        me1=Menu(self.avg_jformat)
        me1.add_command(label="Open", underline=0,
                        command=Press_AVG)

        
        self.avg_jformat.configure(menu=me1)
        
        #### add STN-->DATCOR to Menu Bar ######
        self.stn=Menubutton(self, text="STN-->DATCOR")#,font=('Times',12,"bold"))
        # self.stn.pack(side=LEFT, padx=10)
        self.stn.grid(row=0, column=2, padx=20,pady=7, sticky=W )
        # scrolling of Menu<STN-->DATCOR> 
        me1=Menu(self.stn)
        me1.add_command(label="Open", underline=0,
                        command=Press_STND)

        
        self.stn.configure(menu=me1)
        
        #### add ITER-->OASISto Menu Bar ######
        self.iteroas=Menubutton(self, text="ITER-->OASIS")
        # self.iteroas.pack(side=LEFT, padx=10)
        self.iteroas.grid(row=0, column=3, padx=20,pady=7, sticky=W )
        # scrolling of Menu<ITER-->OASIS> 
        me1=Menu(self.iteroas)
        me1.add_command(label="Open", underline=0,
                        command=Press_OAS)

        self.iteroas.configure(menu=me1)
        
        # Frame.pack(self,expand=YES)
        
                
class Application(Frame):
    
    def __init__(self,boss=None):
        Frame.__init__(self)
        self.master.title("AVG APP")
        self.master.geometry("650x250-0+20")
        self.master.minsize("650","250")
        self.master.iconbitmap("ag2dg-xripg-001.ico")
        
        mBar=MenuBar_EDIR(self)
        mBar.pack()
        # mBar.grid(row=0, column=0, sticky=W)
        self.can = Canvas(self.master, bg ="light grey", height =180,width =520, borderwidth=2,
                          relief=RAISED)
        self.can.pack()
        # self.can.grid(row=1, column=0)
        self.can.create_text(20,70,anchor=NW, text="Welcome to AVG-APPLICATION", 
                             font=('Comic sans MS',20,"italic"),fill="red")
        self.can.create_text(430,165,anchor=NW, text="Version V.00.01", 
                             font=('Comic sans MS',10,"italic"),fill="red")
        
        self.bou=Button(self.master, text="QUIT", font=("bold",10), command=self.master.quit)
        self.bou.pack(side=RIGHT)#height=90, width=500)
        
        
    def clean_can(self):
        self.can.delete(ALL)
    
if __name__=="__main__":
    Application().mainloop()

    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    